<?php
/**
 * Pancake Forms
 * 
 * @author  Ethan Poole
 * @package Pancake
 */

namespace Pancake\Field;

use Pancake\Settings;
use Pancake\Validator\isDate;

/**
 * Three text <input> boxes for the day, month, and year
 * 
 * Examples:
 * <code>
 * $form->date('birthday');
 * $form->date('birthday', array('date_format' => 'Y-m-d'));
 * </code>
 * 
 * Field-specific properties:
 * - <b>container:</b>     OPTIONAL Tag for the group container (default: "<div>")
 * - <b>date_format:</b>   OPTIONAL Date format for formatted value (default: {@link \Pancake\Settings\Dates::DATE_FORMAT})
 * - <b>label_element:</b> OPTIONAL Tag for the group label (default: "<h3>")
 * 
 * @package    Pancake
 * @subpackage Fields
 */
class Date extends Field
{
	public static $methodName = 'date';
	protected $_paragraphClasses = array(
		'date',
		'field'
	);
	
	// Return date format
	protected $_validProperties = array(
		'date_format' => Settings\Dates::DATE_FORMAT
	);
	
	// Group fields into a <div> instead of a <p> and change the label tag
	protected $_properties = array(
		'container' => 'div',
		'label_element' => 'h3'
	);
	
	/**
	 * Child field class name
	 * 
	 * @var string
	 */
	protected $_childFieldClass = 'Pancake\Field\InputText';
	
	/**
	 * Creates the child fields and adds the default date validation
	 */
	protected function _init()
	{		
		// Attributes and properties are set to make sense for entering each piece of the date.
		// The extra hidden text in the label is for screen readers who won't read the grouping label.
		$this->_childFields[$this->name . '_day'] = new $this->_childFieldClass($this->name . '_day', array(
			'class' => 'day',
			'label' => '<span class="hide">' . $this->label(null, false) . ' </span>Day',
			'maxlength' => 2,
			'required' => $this->property('required'),
			'show_if_optional' => false,
			'validation' => 'IsDateDay'
		));
		$this->_childFields[$this->name . '_month'] = new $this->_childFieldClass($this->name . '_month', array(
			'class' => 'month',
			'label' => '<span class="hide">' . $this->label(null, false) . ' </span>Month',
			'maxlength' => 2,
			'required' => $this->property('required'),
			'show_if_optional' => false,
			'validation' => 'IsDateMonth'
		));
		$this->_childFields[$this->name . '_year'] = new $this->_childFieldClass($this->name . '_year', array(
			'class' => 'year',
			'label' => '<span class="hide">' . $this->label(null, false) . ' </span>Year',
			'maxlength' => 4,
			'required' => $this->property('required'),
			'show_if_optional' => false,
			'validation' => 'IsDateYear'
		));
		
		// Add date validation
		$this->_addValidation('IsDate');
	}
	
	/**
	 * Foregoes a default value
	 */
	public function defaultValue()
	{
		
	}
	
	/**
	 * Processes the value
	 * 
	 * The Unix timestamp is broken down into the day, month, and year,
	 * assigning each value to the appropriate child field.  A value is
	 * formed from these child values only when they are all valid.
	 * 
	 * @param  string $value OPTIONAL Unix timestamp
	 * @return string|null   Date string or null if invalid
	 */
	protected function _value($value = null)
	{
		// Set the value of the date inputs
		if ($value !== null)
		{
			// Assume an integer is a valid Unix timestamp.  Process the
			// timestamp and assign the appropriate piece to each child input.
			if (is_int($value))
			{
				$this->_childFields[$this->name . '_day']->value = date('j', $value);
				$this->_childFields[$this->name . '_month']->value = date('n', $value);
				$this->_childFields[$this->name . '_year']->value = date('Y', $value);
			}
			// Reset the child field values otherwise
			else
			{
				$this->_childFields[$this->name . '_day']->value = '';
				$this->_childFields[$this->name . '_month']->value = '';
				$this->_childFields[$this->name . '_year']->value = '';
				
				return parent::_value($value);
			}
		}
		
		// If the child fields are all valid, attempt to create a Unix timestamp based on the values
		if (get_class($this->_childFields[$this->name . '_day']) == $this->_childFieldClass
			&& $this->_childFields[$this->name . '_day']->isValid(false)
			&& $this->_childFields[$this->name . '_month']->isValid(false)
			&& $this->_childFields[$this->name . '_year']->isValid(false))
		{
			$value = mktime(0, 0, 0, $this->_childFields[$this->name . '_month']->value, $this->_childFields[$this->name . '_day']->value, $this->_childFields[$this->name . '_year']->value);
			parent::_value($value);
			return date(isDate::DATE_FORMAT, $this->_value);
		}
		// Otherwise, return nothing
		else
		{
			return null;
		}
	}
	
	/**
	 * Returns a formatted date
	 * 
	 * @return string
	 */
	protected function _formattedValue()
	{
		return (!empty($this->_value)) ? date($this->property('date_format'), $this->_value) : null;
	}
}