<?php
/**
 * Pancake Forms
 * 
 * @author  Ethan Poole
 * @package Pancake
 */

namespace Pancake\Field;

use Pancake\Settings;
use Pancake\Validator\IsDate;

/**
 * Three <select> drop downs for the day, month, and year
 * 
 * Examples:
 * <code>
 * $form->dateDropDowns('birthday');
 * $form->dateDropDowns('birthday', array('year_start' => '1900', 'year_end' => 'today'));
 * </code>
 * 
 * Field-specific properties:
 * - <b>container:</b>     OPTIONAL Tag for the group container (default: "<div>")
 * - <b>date_format:</b>   OPTIONAL Date format for formatted value (default: {@link \Pancake\Settings\Dates::DATE_FORMAT})
 * - <b>label_element:</b> OPTIONAL Tag for the group label (default: "<h3>")
 * - <b>year_end:</b>      OPTIONAL End year, run through {@link \strtotime()} (default: "+5 years")
 * - <b>year_start:</b>    OPTIONAL Start year, run through {@link \strtotime()} (default: "-5 years")
 * 
 * @package    Pancake
 * @subpackage Fields
 */
class DateDropDowns extends Field
{
	public static $methodName = 'dateDropDowns';
	protected $_paragraphClasses = array(
		'date',
		'date-drop-downs',
		'field'
	);
	
	protected $_validProperties = array(
		// Start and end for the year range
		'year_start' => '-5 years',
		'year_end' => '+5 years',
		
		// Return date format
		'date_format' => Settings\Dates::DATE_FORMAT
	);
	
	// Group fields into a <div> instead of a <p> and change the label tag
	protected $_properties = array(
		'container' => 'div',
		'label_element' => 'h3'
	);
	
	/**
	 * Child field class name
	 * 
	 * @var string
	 */
	protected $_childFieldClass = 'Pancake\Field\Select';
	
	/**
	 * Pairing of month strings to integer values
	 * 
	 * @var array
	 */
	protected static $_months = array(
		1 => 'January',
		2 => 'February',
		3 => 'March',
		4 => 'April',
		5 => 'May',
		6 => 'June',
		7 => 'July',
		8 => 'August',
		9 => 'September',
		10 => 'October',
		11 => 'November',
		12 => 'December'
	);
	
	/**
	 * Creates the child fields and adds the default date validation
	 */
	protected function _init()
	{
		// Determine the range of years to show
		$yearStart = getdate(strtotime($this->property('year_start')));
		$yearStart = $yearStart['year'];
		$yearEnd = getdate(strtotime($this->property('year_end')));
		$yearEnd = $yearEnd['year'];
		
		// Day, month, and year options
		$days = range(1, 31);
		$months = DateDropDowns::$_months;
		$years = range($yearStart, $yearEnd);
		
		// Attributes and properties are set to make sense for entering each piece of the date
		$this->_childFields[$this->name . '_day'] = new $this->_childFieldClass(
			$this->name . '_day',
			$days,
			array(
				'class' => 'day',
				'required' => $this->property('required'),
				'show_if_optional' => false
			)
		);
		$this->_childFields[$this->name . '_month'] = new $this->_childFieldClass(
			$this->name . '_month',
			$months,
			array(
				'class' => 'month',
				'required' => $this->property('required'),
				'show_if_optional' => false
			)
		);
		$this->_childFields[$this->name . '_year'] = new $this->_childFieldClass(
			$this->name . '_year',
			$years,
			array(
				'class' => 'year',
				'required' => $this->property('required'),
				'show_if_optional' => false
			)
		);
		
		// Add date validation
		$this->_addValidation('IsDate');
	}
	
	/**
	 * Foregoes a default value
	 */
	public function defaultValue()
	{
		
	}
	
	/**
	 * Processes the value
	 * 
	 * The Unix timestamp is broken down into the day, month, and year,
	 * assigning each value to the appropriate child field.  A value is
	 * formed from these child values only when they are all valid.
	 * 
	 * @param  string $value OPTIONAL Unix timestamp
	 * @return string|null   Date string or null if invalid
	 */
	protected function _value($value = null)
	{
		// Set the value of the date inputs
		if ($value !== null)
		{
			// Assume an integer is a valid Unix timestamp.  Process the
			// timestamp and assign the appropriate piece to each child input.
			if (is_int($value))
			{
				$this->_childFields[$this->name . '_day']->value = date('j', $value);
				$this->_childFields[$this->name . '_month']->value = date('F', $value);
				$this->_childFields[$this->name . '_year']->value = date('Y', $value);
			}
			// Reset the child field values otherwise
			else
			{
				$this->_childFields[$this->name . '_day']->value = '';
				$this->_childFields[$this->name . '_month']->value = '';
				$this->_childFields[$this->name . '_year']->value = '';
				
				return parent::_value($value);
			}
		}
		
		// If the child fields are all valid, attempt to create a Unix timestamp based on the values
		if (get_class($this->_childFields[$this->name . '_day']) == $this->_childFieldClass
			&& $this->_childFields[$this->name . '_day']->isValid(false)
			&& $this->_childFields[$this->name . '_month']->isValid(false)
			&& $this->_childFields[$this->name . '_year']->isValid(false))
		{
			$value = mktime(0, 0, 0, array_search(ucfirst($this->_childFields[$this->name . '_month']->value[0]), DateDropDowns::$_months), $this->_childFields[$this->name . '_day']->value[0], $this->_childFields[$this->name . '_year']->value[0]);
			parent::_value($value);
			return date(IsDate::DATE_FORMAT, $this->_value);
		}
		// Otherwise, return nothing
		else
		{
			return null;
		}
	}
	
	/**
	 * Returns a formatted date
	 * 
	 * @return string
	 */
	protected function _formattedValue()
	{
		return (!empty($this->_value)) ? date($this->property('date_format'), $this->_value) : null;
	}
}