<?php
/**
 * Pancake Forms
 * 
 * @author  Ethan Poole
 * @package Pancake
 */

namespace Pancake\Field;

use Pancake\Settings;
use Pancake\Validator\IsSemester;

/**
 * Two <select> drop downs for semester and year
 * 
 * Examples:
 * <code>
 * $form->dateEstimation('graduation');
 * $form->dateEstimation('graduation', array('year_start' => '1900', 'year_end' => '+10 years'));
 * </code>
 * 
 * Field-specific properties:
 * - <b>container:</b>     OPTIONAL Tag for the group container (default: "<div>")
 * - <b>label_element:</b> OPTIONAL Tag for the group label (default: "<h3>")
 * - <b>year_end:</b>      OPTIONAL End year, run through {@link \strtotime()} (default: "+5 years")
 * - <b>year_start:</b>    OPTIONAL Start year, run through {@link \strtotime()} (default: "-5 years")
 * 
 * @package    Pancake
 * @subpackage Fields
 */
class DateEstimation extends Field
{
	public static $methodName = 'dateEstimation';
	protected $_paragraphClasses = array(
		'date',
		'date-drop-downs',
		'field'
	);
	
	// Start and end for the year range
	protected $_validProperties = array(
		'year_start' => '-5 years',
		'year_end' => '+5 years'
	);
	
	// Group fields into a <div> instead of a <p> and change the label tag
	protected $_properties = array(
		'container' => 'div',
		'label_element' => 'h3'
	);
	
	/**
	 * Child field class name
	 * 
	 * @var string
	 */
	protected $_childFieldClass = 'Pancake\Field\Select';
	
	/**
	 * Creates the child fields and adds the default validation
	 */
	protected function _init()
	{
		// Determine the range of years to show
		$yearStart = getdate(strtotime($this->property('year_start')));
		$yearStart = $yearStart['year'];
		$yearEnd = getdate(strtotime($this->property('year_end')));
		$yearEnd = $yearEnd['year'];
		$years = range($yearStart, $yearEnd);
		
		// Attributes and properties are set to make sense for entering each piece of the date estimation
		$this->_childFields[$this->name . '_semester'] = new $this->_childFieldClass(
			$this->name . '_semester',
			IsSemester::$semesters,
			array(
				'class' => 'semester',
				'required' => $this->property('required'),
				'show_if_optional' => false
			)
		);
		$this->_childFields[$this->name . '_year'] = new $this->_childFieldClass(
			$this->name . '_year',
			$years,
			array(
				'class' => 'year',
				'required' => $this->property('required'),
				'show_if_optional' => false
			)
		);
		
		// Use a simple regular expression to validate the overall value
		$this->_addValidation(array(
			'MatchesPattern',
			Settings\Dates::SEMESTER_REGEX,
			'error' => 'Please enter a valid date.'
		));
	}
	
	/**
	 * Foregoes a default value
	 */
	public function defaultValue()
	{
		
	}
	
	/**
	 * Processes the value
	 * 
	 * A regex match breaks up the value into two pieces, each of which is
	 * assigned to the respective field.  A value is formed from these child
	 * fields only when they are all valid.
	 * 
	 * @param  string $value OPTIONAL
	 * @return string|null
	 */
	protected function _value($value = null)
	{
		// Set the values of the individual <select> drop downs
		if ($value !== null)
		{
			// Check for a proper regular-expression match
			if (preg_match(Settings\Dates::SEMESTER_REGEX, $value, $matches))
			{
				$this->_childFields[$this->name . '_semester']->value = $matches[1];
				$this->_childFields[$this->name . '_year']->value = $matches[2];
			}
			// Reset the child field values otherwise
			else
			{
				$this->_childFields[$this->name . '_semester']->value = '';
				$this->_childFields[$this->name . '_year']->value = '';
				
				return parent::_value($value);
			}
		}
		
		// If the child fields are all valid, form an overall value for the field
		if (get_class($this->_childFields[$this->name . '_semester']) == $this->_childFieldClass
			&& $this->_childFields[$this->name . '_semester']->isValid(false)
			&& $this->_childFields[$this->name . '_year']->isValid(false))
		{
			$value = $this->_childFields[$this->name . '_semester']->value[0] . ' ' . $this->_childFields[$this->name . '_year']->value[0];
			return parent::_value($value);
		}
		// Otherwise, return nothing
		else
		{
			return null;
		}
	}
	
	/**
	 * Returns a formatted overall value
	 * 
	 * @return string
	 */
	protected function _formattedValue()
	{
		return ucfirst($this->_childFields[$this->name . '_semester']->value[0]) . ' ' . $this->_childFields[$this->name . '_year']->value[0];
	}
}