<?php
/**
 * Pancake Forms
 * 
 * @author  Ethan Poole
 * @package Pancake
 */

namespace Pancake\Field;

use Pancake\Exception;
use Pancake\Helper;
use Pancake\Settings;
use Pancake\Validator;

/**
 * Base form field class
 * 
 * Form fields are used to collect data from the user.  This class provides
 * a centralised framework for processing properties and attributes; assigning
 * and validating data; and returning string outputs.
 * 
 * See properties valid for all fields in {@link \Pancake\Field\Field::$_uberValidProperties}.
 * 
 * See attributes valid for all fields in {@link \Pancake\Field\Field::$_uberValidAttributes}.
 * 
 * @package    Pancake
 * @subpackage Fields
 */
abstract class Field
{
	/**
	 * Unique field name
	 * 
	 * Can contain only alphanumeric characters, underscores, and dashes.
	 * 
	 * @var string
	 */
	protected $_name;
	
	/**
	 * Properties
	 * 
	 * Properties are settings for fields that are <b>not</b> HTML
	 * attributes, such as whether to hide the label, show help
	 * text, require the field, etc.
	 * 
	 * Get and set properties through {@link \Pancake\Field\Field::property()}.
	 * 
	 * @var array
	 * @see \Pancake\Field\Field::property()
	 */
	protected $_properties = array();
	
	/**
	 * Attributes
	 * 
	 * Attributes are settings for the field that <b>are</b>
	 * HTML attributes, such as "class" or "id".
	 * 
	 * Get and set attributes through {@link \Pancake\Field\Field::attribute()}.
	 * 
	 * @var array
	 * @see \Pancake\Field\Field::attribute()
	 */
	protected $_attributes = array();
	
	/**
	 * Classes for the field's paragraph
	 * 
	 * @var array
	 */
	protected $_paragraphClasses;
	
	/**
	 * Value of the field
	 * 
	 * Set and get the field value through {@link \Pancake\Field\Field::_value()}.
	 * 
	 * @var mixed
	 * @see \Pancake\Field\Field::_value()
	 */
	protected $_value;
	
	/**
	 * Validation functions
	 * 
	 * Add a validation function through {@link \Pancake\Field\Field::_addValidation()}.
	 * 
	 * @var array
	 * @see \Pancake\Field\Field::_addValidation()
	 */
	protected $_validation = array();
	
	/**
	 * Error messages
	 * 
	 * @var array
	 */
	protected $_errors = array();
	
	/**
	 * Child fields
	 * 
	 * Child fields are useful for dates or other single entities that
	 * are better entered by the user through multiple form fields.
	 * 
	 * @var array
	 */
	protected $_childFields = array();
	
	/**
	 * Automate name and ID attributes
	 * 
	 * Since most fields have "id" and "name" attributes, this can be
	 * automated, yet configurable just in case (i.e. <option> does not).
	 * 
	 * @var boolean
	 */
	protected $_autoNameAndId = true;
	
	/**
	 * Whether the field can be the form's identifier
	 * 
	 * Form identifiers are used to differentiate POST data between multiple
	 * forms that are on the same page.  By default, submit buttons will act
	 * as form identifiers.
	 * 
	 * @var boolean
	 */
	protected $_canBeFormIdentifier = false;
	
	/**
	 * Properties of all fields, with defaults
	 * 
	 * Über properties are applicable to all form fields and should not be
	 * overwritten in sub-classes.
	 * 
	 * Properties:
	 * - <b>accessible_text:</b> Instructional text intended for visually-impaired users
	 * - <b>after:</b> After the field
	 * - <b>before:</b> Before the field
	 * - <b>container:</b> HTML container tag
	 * - <b>label:</b> Label text
	 * - <b>label_element:</b> HTML label tag ("for" attribute will be removed if not "label")
	 * - <b>label_first:</b> Whether the label or the field is first
	 * - <b>label_short:</b> Short label text
	 * - <b>help_text:</b> Help text
	 * - <b>paragraph:</b> Whether to encompass the field in a paragraph
	 * - <b>paragraph_classes:</b> Classes added to the contain paragraph
	 * - <b>required:</b> Whether the field is required
	 * - <b>required_if:</b> Field name and value that makes the field required
	 * - <b>show_errors:</b> Whether to show validation errors
	 * - <b>show_field:</b> Whether to show the form field
	 * - <b>show_if_optional:</b> Whether to show if the field is optional
	 * - <b>show_label:</b> Whether to show the label
	 * 
	 * @var array
	 */
	protected $_uberValidProperties = array(
		'accessible_text' => null,
		'after' => null,
		'before' => null,
		'container' => 'p',
		'label' => null,
		'label_element' => 'label',
		'label_first' => true,
		'label_short' => null,
		'help_text' => null,
		'paragraph' => true,
		'paragraph_classes' => null,
		'required' => true,
		'required_if' => null,
		'show_errors' => true,
		'show_field' => true,
		'show_if_optional' => true,
		'show_label' => true
	);
	
	/**
	 * Valid field-specific properties
	 * 
	 * Sub-classes should specify properties specific to that field,
	 * along with providing default values.
	 * 
	 * @var array
	 */
	protected $_validProperties = array();
	
	/**
	 * Attributes of all fields, with defaults
	 * 
	 * Über attributes are application to all form fields and should not be
	 * overwritten in sub-classes.
	 * 
	 * @var array
	 */
	protected $_uberValidAttributes = array(
		'class' => null,
		'id' => null,
		'name' => null
	);
	
	/**
	 * Valid field-specific attributes
	 * 
	 * Sub-classes should specify attributes specific to that field,
	 * along with providing default values.
	 * 
	 * @var array
	 */
	protected $_validAttributes = array();
	
	/**
	 * Required properties and attributes
	 * 
	 * @var array
	 */
	protected $_requiredParameters = array();
	
	/**
	 * Sifts through the parameters and builds the field instance
	 * 
	 * The parameters should always start with the field's name, follow with any required attributes (cols, rows, etc.),
	 * and then conclude with an array of optional properties and attributes.  If any parameters remain after this
	 * processing, an exception is thrown.
	 * 
	 * Properties and attributes are merged with the über settings, the default field settings, and then
	 * anything set at runtime.
	 * 
	 * Examples:
	 * <code>
	 * $form->input('name', 'text');
	 * $form->text('name');
	 * $form->textarea('name', 'cols', 'rows', array('property' => 'value', 'attribute' => 'value'));
	 * </code>
	 * 
	 * @param  string $name                  Field name containing only alphanumeric characters, underscores, and dashes
	 * @param  mixed  $requiredParameter,...
	 * @param  array  $optionalParameters    OPTIONAL
	 * 
	 * @throws \Pancake\Exception\ValueError      When the first parameter does not exist, is not a string, or contains invalid characters
	 * @throws \Pancake\Exception\MissingArgument When the attribute count is below the required amount
	 * @throws \Pancake\Exception\Exception       When there are leftover parameters (i.e. invalid parameters)
	 */
	public function __construct()
	{
		// Get parameters, which may be a single array if from Pancake::addField()
		$parameters = func_get_args();
		if (count($parameters) == 1 && is_array($parameters[0]))
		{
			$parameters = $parameters[0];
		}
		
		// Make sure there is a first parameter that is the field's name
		if (empty($parameters[0]) || !is_string($parameters[0]) || !preg_match('#^[a-zA-Z0-9_]*$#', $parameters[0]))
		{
			throw new Exception\ValueError('The first field parameter, its name, must be a string containing only alphanumeric characters, dashes, and underscores.');
		}
		
		// Field name
		$this->_name = (string) array_shift($parameters);
		
		// Process required properties and attributes
		foreach ($this->_requiredParameters as $key => $parameter)
		{
			// Decide if the parameter is an attribute or a property
			if (array_key_exists($parameter, $this->_validProperties) || array_key_exists($parameter, $this->_uberValidProperties))
			{
				$settings = &$this->_properties;
			}
			elseif ((array_key_exists($parameter, $this->_validAttributes) || array_key_exists($parameter, $this->_uberValidAttributes)))
			{
				$settings = &$this->_attributes;
			}
			
			// Do *not* override field defaults at this level
			if (!array_key_exists($parameter, $settings))
			{
				if (!array_key_exists($key, $parameters))
				{
					throw new Exception\MissingArgument(count($this->_requiredParameters) . ' parameters are required for ' . get_class($this));
					return false;
				}
				else
				{
					$settings[$parameter] = $parameters[$key];
					unset($parameters[$key]);
				}
			}
		}
		
		// Automate the assignment of the name and ID attributes
		// \Pancake\Field\Field::attribute() cannot be used since the attribute do not yet exist
		if ($this->_autoNameAndId === true)
		{
			$this->_attributes['id'] = (string) $this->id;
			$this->_attributes['name'] = (string) $this->name;
		}
		
		// Default properties and attributes
		$this->_properties = array_merge($this->_uberValidProperties, $this->_validProperties, $this->_properties);
		$this->_attributes = array_merge($this->_uberValidAttributes, $this->_validAttributes, $this->_attributes);
		
		// Process optional attributes
		// array_key_exists() is used in case the values are intentionally set to null (since isset() would return false)
		if (count($parameters) == 1)
		{
			$parameters = array_pop($parameters);
			
			if (is_array($parameters))
			{
				// Validation function
				if (array_key_exists('validation', $parameters))
				{
					// Validator with parameters
					if (is_array($parameters['validation']))
					{
						foreach ($parameters['validation'] as $validation)
						{
							$this->_addValidation($validation);
						}
					}
					// Plain validator
					else
					{
						$this->_addValidation($parameters['validation']);
					}
					
					unset($parameters['validation']);
				}
				
				// Field value
				// Save the field value such that it can be reprocessed if
				// the field has child fields.
				if (array_key_exists('value', $parameters))
				{
					$value = $parameters['value'];
					$this->value = $value;
					unset($parameters['value']);
				}
				
				// Properties
				$properties = array_intersect_key($parameters, $this->_properties);
				if (!empty($properties))
				{
					$this->_properties = array_merge($this->_properties, $properties);
					$parameters = array_diff_assoc($parameters, $properties);
				}
				
				// Attributes
				$attributes = array_intersect_key($parameters, $this->_attributes);	
				if (!empty($attributes))
				{
					$this->_attributes = array_merge($this->_attributes, $attributes);
					$parameters = array_diff_assoc($parameters, $attributes);
				}
			}
		}
		
		// Make sure that every parameter has been accounted for
		if (!empty($parameters))
		{
			throw new Exception\Exception(count($parameters) . ' field parameters leftover.');
		}
		
		// Mark conditionally required fields as not required by default
		if ($this->property('required_if') !== null)
		{
			$this->property('required', false);
		}
		
		// Run the field-specific constructor
		$this->_init();
		
		// Re-run _value() if _init() created child fields that may receive
		// their values from the parent's _value() method.
		if (isset($value) && count($this->childFields) > 0)
		{
			$this->value = $value;
		}
	}
	
	/**
	 * Field-specific constructor
	 * 
	 * Called after {@link \Pancake\Field\Field::__construct()}.
	 */
	protected function _init()
	{
		
	}
	
	/**
	 * Field output
	 * 
	 * This method should be overwritten in sub-classes for each form field.
	 * However, this method provides some default handling for child fields
	 * by concatenation their output together.
	 * 
	 * @return string
	 */
	public function output()
	{
		// Default handling for child fields
		if (count($this->childFields) > 0)
		{
			$return = null;
			foreach ($this->childFields as $field)
			{
				$return .= $field->__toString();
			}
			
			return $return;
		}
		else
		{
			return null;
		}
	}
	
	/**
	 * Produces the <label> tag for the field
	 * 
	 * @param  string  $text        OPTIONAL Runtime text
	 * @param  boolean $includeTags OPTIONAL Whether to return just the text or a formatted <label> tag
	 * @param  array   $attributes  OPTIONAL <label> tag attributes
	 * @return string
	 */
	public function label($text = null, $includeTags = true, $attributes = array())
	{
		if ($text == null)
		{
			// Use the label property
			if ($this->property('label') != null)
			{
				$text = $this->property('label');
			}
			// Default text based on the field name
			else
			{
				$text = Helper\Readable::process($this->name);
			}
		}
		
		// Return the <label> tag
		if ($includeTags === true)
		{
			// Optionality message
			if ($this->property('required') === false && $this->property('show_if_optional') === true)
			{
				$text .= Settings\Fields::OPTIONAL_HTML;
			}
			
			// Error messages
			if ($this->property('show_errors') === true)
			{
				foreach ($this->_errors as $error)
				{
					$text .= sprintf(Settings\Fields::ERROR_HTML, $error);
				}
			}
			
			// Help text
			if ($this->property('help_text') != null)
			{
				$text .= sprintf(Settings\Fields::HELP_TEXT_HTML, $this->property('help_text'));
			}
			
			// Accessible text
			if ($this->property('accessible_text') != null)
			{
				$text .= sprintf(Settings\Fields::ACCESSIBLE_TEXT_HTML, $this->property('accessible_text'));
			}
			
			// If the label element is <label>, then set up the correct "for" attribute
			if ($this->property('label_element') == 'label')
			{
				$defaultAttributes = array(
					'for' => $this->id
				);
				$validAttributes = array('class', 'for', 'id', 'title');
			}
			// Otherwise, assume very little about the element
			else
			{
				$defaultAttributes = array();
				$validAttributes = array('class', 'id', 'title');
			}
			
			return '<' . $this->property('label_element') . Helper\Attributes::process($attributes, $defaultAttributes, $validAttributes) . '>' . $text . '</' . $this->property('label_element') . '>';
		}
		// Return just the label text
		else
		{
			return $text;
		}
	}
	
	/**
	 * Runs the field value through the validation functions
	 * 
	 * @param  boolean $storeErrors OPTIONAL Whether or not to store the errors after validation
	 * @return boolean                       Whether or not the field's value is valid
	 */
	public function isValid($storeErrors = true)
	{
		// Run through validation functions
		foreach ($this->_validation as $validation)
		{
			array_unshift($validation['parameters'], $this->value);
			$return = call_user_func_array($validation['validator'], $validation['parameters']);
			
			// Save the error message (custom or default)
			if (is_string($return))
			{
				$this->_errors[] = ($validation['error'] != null) ? $validation['error'] : $return;
			}
		}
		
		// Check if the field is required if no other error
		if ($this->property('required') && empty($this->_errors))
		{
			$isRequired = new Validator\IsRequired;
			
			$return = $isRequired->validate($this->value);	
			if ($return !== true)
			{
				$this->_errors[] = $return;
			}
		}
		// Allow non-required fields to be blank
		elseif (!$this->property('required') && $this->value == null)
		{
			$this->_errors = array();
		}
		
		// Keep the errors
		if ($storeErrors !== false)
		{
			return empty($this->_errors);
		}
		// Empty the errors before returning true/false
		else
		{
			$return = empty($this->_errors);
			$this->_errors = array();
			return $return;
		}
	}
	
	/**
	 * Sets the default value for the field
	 * 
	 * By default, this method does very little, but in sub-classes it
	 * can be overwritten to set a more customised value or to forego
	 * a default value all together.
	 */
	public function defaultValue()
	{
		$this->value = '';
	}
	
	/**
	 * Sets and gets a field property
	 * 
	 * @param  string $key                     Property name
	 * @param  mixed  $value                   OPTIONAL Value to assign the property
	 * @return mixed
	 * @throws \Pancake\Exception\DoesNotExist When the property does not exist
	 */
	public function property($key, $value = null)
	{
		if (array_key_exists($key, $this->_properties))
		{
			if ($value !== null)
			{
				$this->_properties[$key] = $value;
			}
			
			return $this->_properties[$key];
		}
		else
		{
			throw new Exception\DoesNotExist('The property "' . $key . '" does not exist.');
			return false;
		}
	}
	
	/**
	 * Sets and gets a field attribute
	 * 
	 * @param  string $key                     Attribute name
	 * @param  string $value                   OPTIONAL Value to assign the attribute
	 * @return string|booleans
	 * @throws \Pancake\Exception\DoesNotExist When the attribute does not exist
	 */
	public function attribute($key, $value = null)
	{
		if (array_key_exists($key, $this->_attributes))
		{
			if ($value !== null)
			{
				$this->_attributes[$key] = $value;
			}
			
			return $this->_attributes[$key];
		}
		else
		{
			throw new Exception\DoesNotExist('The attribute "' . $key . '" does not exist.');
			return false;
		}
	}
	
	/**
	 * Returns the field's ID
	 * 
	 * @return string
	 */
	protected function _id()
	{
		return Settings\Fields::ID_PREFIX . $this->name;
	}
	
	/**
	 * Sets and gets the field's value
	 * 
	 * By default, this method does very little, but in sub-classes it
	 * can be overwritten to automate the <b>display</b> of a field's value, via 
	 * an attribute or something else.  However, it is useful to call this
	 * function to ensure {@link \Pancake\Field\Field::$_value} is always set properly.
	 * 
	 * @param  mixed $value OPTIONAL
	 * @return mixed
	 */
	protected function _value($value = null)
	{
		if ($value !== null)
		{
			$this->_value = $value;
		}
		
		return $this->_value;
	}
		
	/**
	 * Returns the field's formatted value
	 * 
	 * Sub-classes can overwrite this method if the human readable version
	 * of the field's data requires any parsing.
	 * 
	 * @return mixed
	 */
	protected function _formattedValue()
	{
		return $this->value;
	}
	
	/**
	 * Adds a validation function
	 * 
	 * Examples:
	 * <code>
	 * \Pancake\Field\Field::_addValidation('user_defined_function');
	 * \Pancake\Field\Field::_addValidation('isEmail');
	 * \Pancake\Field\Field::_addValidation(array('isBetween', 0, 4));
	 * </code>
	 * 
	 * Included validation libraries should use only the class name.
	 * 
	 * @param  string|array $validation     Function name or an array with the function name and parameters
	 * @return boolean                      Whether or not the validation function was added successfully
	 * @throws \Pancake\Exception\Exception When the validator function does not exist
	 */
	protected function _addValidation($validation)
	{
		// Validation function with parameters
		if (is_array($validation))
		{
			// Allow a custom error message to be set
			$error = (isset($validation['error'])) ? $validation['error'] : null;
			
			$validation = array(
				'validator' => ucfirst(array_shift($validation)),
				'parameters' => $validation,
				'error' => $error
			);
		}
		else
		{
			$validation = array(
				'validator' => ucfirst($validation),
				'parameters' => array(),
				'error' => null
			);
		}
		
		// Included validation library
		if (file_exists(PANCAKE_ROOT . 'Validator/' . $validation['validator'] . '.php'))
		{
			$className = '\Pancake\Validator\\' . $validation['validator'];
			$validation['validator'] = array(new $className, 'validate');
		}
		// Ensure the user-defined function exists
		elseif (!function_exists($validation['validator']))
		{
			throw new Exception\Exception('The validation function "' . $validation['validator'] . '()" does not exist.');
			return false;
		}
		elseif (function_exists($validation['validator']))
		{
			$validation['validator'] = $validation['validator'];
		}
		
		$this->_validation[] = $validation;
		return true;
	}
	
	/**
	 * Clears all of the validation functions
	 */
	protected function _clearValidation()
	{
		$this->_validation = array();
	}
	
	/**
	 * Magic getters
	 * 
	 * @param  string $member
	 * @return mixed
	 */
	public function __get($member)
	{
		switch ($member)
		{
			case 'name':
				return $this->_name;
			break;
			
			case 'errors':
				return $this->_errors;
			break;
			
			case 'canBeFormIdentifier':
				return $this->_canBeFormIdentifier;
			break;
			
			case 'childFields':
				return $this->_childFields;
			break;
			
			case 'value':
				return $this->_value();
			break;
			
			case 'formattedValue':
				return $this->_formattedValue();
			break;
			
			case 'id':
				return $this->_id();
			break;
			
			case 'label':
				return $this->label();
			break;
		}
	}
	
	/**
	 * Magic setters
	 * 
	 * @param  string $member
	 * @param  mixed  $value
	 * @return mixed
	 */
	public function __set($member, $value)
	{
		switch ($member)
		{
			case 'value':
				return $this->_value($value);
			break;
		}
	}
	
	/**
	 * Converts the field into a string
	 * 
	 * @return string Nicely formatted field
	 */
	public function __toString()
	{
		$output = null;
		
		// Label
		if ($this->property('show_label') === true)
		{
			$output .= (string) $this->label();
		}
		
		// Field
		if ($this->property('show_field') === true)
		{
			// Show label before or after the field?
			if ($this->property('label_first') === false)
			{
				$output = (string) $this->output() . $output;
			}
			else
			{
				$output .= (string) $this->output();
			}
		}
		
		// Paragraph container
		if ($this->property('paragraph') === true)
		{
			// Add user-specified paragraph classes
			if (is_array($this->property('paragraph_classes')))
			{
				$this->_paragraphClasses = array_merge($this->_paragraphClasses, $this->property('paragraph_classes'));
			}
			
			// Field-state paragraph classes (error, optionality, etc.)
			$additionalClasses = array(
				Settings\Fields::ERROR_CLASS => (!empty($this->_errors)),
				Settings\Fields::OPTIONAL_CLASS => ($this->property('required') === false),
				Settings\Fields::HELP_TEXT_CLASS => ($this->property('help_text') != null),
				Settings\Fields::ACCESSIBLE_TEXT_CLASS => ($this->property('accessible_text') != null)
			);
			foreach ($additionalClasses as $class => $add)
			{
				if ($add && !in_array($class, $this->_paragraphClasses))
				{
					$this->_paragraphClasses[] = $class;
				}
			}
			
			// Before and after the field
			if ($this->property('before'))
			{
				$output = $this->property('before') . $output;
			}
			if ($this->property('after'))
			{
				$output .= $this->property('after');
			}
			
			// Process paragraph classes
			$paragraphClasses = null;
			if (!empty($this->_paragraphClasses))
			{
				sort($this->_paragraphClasses);
				$paragraphClasses = implode(' ', $this->_paragraphClasses);
			}
			
			$output = '<' . $this->property('container')  . Helper\Attributes::process(array('class' => $paragraphClasses)) . '>' . $output . '</' . $this->property('container') . '>';
		}
		
		return $output;
	}
}