<?php
/**
 * Pancake Forms
 * 
 * @author  Ethan Poole
 * @package Pancake
 */

namespace Pancake\Field;

use Pancake\Exception;
use Pancake\Helper;
use Pancake\Validator\IsOneOf;

/**
 * Base list field class
 * 
 * This is the base class for any field containing a series of options,
 * such as a <select> box or a list of radio buttons/checkboxes.  The class
 * is intended to provide a very basic structure for processing options.
 * 
 * Field-specific properties:
 * - <b>options:</b> REQUIRED List options
 * 
 * @package    Pancake
 * @subpackage Fields
 */
abstract class FieldList extends Field
{
	// Require options for the list
	protected $_validProperties = array(
		'options' => array()
	);
	protected $_requiredParameters = array(
		'options'
	);
	
	/**
	 * Class to use for list options
	 * 
	 * The option class must be able to respond to a boolean value, generally
	 * setting a parameter like "checked" or "selected", and to the field's
	 * name being passed as the second parameter.
	 * 
	 * @var string
	 */
	protected $_optionClass;
	
	/**
	 * Whether or not to allow multiple selections
	 * 
	 * @var boolean
	 */
	protected $_allowMultiples = true;
	
	/**
	 * Creates the options and sets up automated validation
	 * 
	 * @throws \Pancake\Exception\MissingArgument When no options are present
	 * @throws \Pancake\Exception\Exception       When a custom option is not formed properly
	 */
	protected function _init()
	{
		$options = $this->property('options');
		
		// Make sure that there are options
		if (empty($options))
		{
			
			throw new Exception\MissingArgument('At least one option is required for FieldList.');
		}
		
		// Formulate the parent name, taking into account FieldList::$_allowMultiples
		$parentName = ($this->_allowMultiples) ? $this->name . '[]' : $this->name;
		
		// Create the list options
		foreach ($options as $option)
		{
			// Custom option with a parameters array
			if (is_array($option))
			{
				// Make sure there are not too many parameters
				if (count($option) == 2)
				{
					$name = $option[0];
					$parameters = $option[1];
				}
				else
				{
					throw new Exception\Exception('A custom option must only be the field\'s name and an array of parameters.');
				}
			}
			// Simple option (name only)
			else
			{
				$name = $option;
				$parameters = array();
			}
			
			// Preserve the name as the attribute and make sure the name is safe
			if (!isset($parameters['label']))
			{
				$parameters['label'] = Helper\Readable::process($name);
			}
			$name = Helper\SafeFieldName::process($name);
			
			// Notify the sub-class of an "other" field
			if ($name == 'other' || $name == '{other}')
			{
				$this->_childFields[$name] = $this->_other($name, $parentName, $parameters);
			}
			else
			{
				$this->_childFields[$name] = new $this->_optionClass($name, $parentName, $parameters);
			}
		}
		
		// Default validation to make sure the value is within the range of choices
		// Use the "multiple" options error message when applicable
		if ($this->_allowMultiples)
		{
			$this->_addValidation(array('IsOneOf', array_keys($this->childFields), 'error' => isOneOf::MULTIPLE_OPTIONS_ERROR));
		}
		else
		{
			$this->_addValidation(array('IsOneOf', array_keys($this->childFields)));
		}
	}
	
	/**
	 * Handles "other" fields
	 * 
	 * Generic method meant to be overwritten by lists that can
	 * have "other" fields.  It should return a form field.
	 * 
	 * @param  string $name       Field name
	 * @param  string $parentName Parent name
	 * @param  array  $parameters Field parameters
	 * @return object             Form field
	 */
	protected function _other($name, $parentName, $parameters)
	{
		return new $this->_optionClass($name, $parentName, $parameters);
	}
	
	/**
	 * Alerts option fields if they have been selected
	 * 
	 * @param  array|string $value OPTIONAL Either a string value or an array of values
	 * @return array
	 */
	protected function _value($value = null)
	{
		if ($value !== null)
		{
			// Make sure $value is an array
			if (!is_array($value))
			{
				$value = array($value);
			}
			
			// Make sure the values are in the correct format or they may
			// not match properly with the options' values.
			$value = array_map('\Pancake\Helper\SafeFieldName::process', $value);
			
			// Find corresponding option to mark as checked (or unchecked)
			foreach ($this->childFields as $option)
			{
				if (in_array($option->attribute('value'), $value))
				{
					$option->value = true;
				}
				else
				{
					$option->value = false;
				}
			}
			
			// Return null if the set contains only one empty element. This is
			// mainly for <select> drop downs who have a blank initial option.
			if (count($value) == 1 && $value[0] == null)
			{
				return parent::_value('');
			}
			else
			{
				return parent::_value($value);
			}
		}
		
		return $this->_value;
	}
	
	/**
	 * Returns a comma-separated list of the options' values
	 * 
	 * @return string
	 */
	protected function _formattedValue()
	{
		$return = array();
		
		foreach ($this->childFields as $option)
		{
			if ($option->value === true)
			{
				$return[] = $option->formattedValue;
			}
		}
		
		return implode(', ', $return);
	}
}