<?php
/**
 * Pancake Forms
 * 
 * @author  Ethan Poole
 * @package Pancake
 */

namespace Pancake\Field;

use Pancake\Helper;
use Pancake\Settings;
use Pancake\Exception;

/**
 * File upload <input> field
 * 
 * Examples:
 * <code>
 * $form->fileUpload('resume');
 * $form->fileUpload('resume', array('upload_to' => '/var/www/html/sua/uploads'));
 * $form->fileUpload('resume', array('label' => 'Résumé', 'upload_to' => '/var/www/html/sua/uploads'));
 * </code>
 * 
 * Field-specific properties:
 * - <b>show_upload_message:</b> OPTIONAL Whether or not to show the upload message (default: true)
 * - <b>upload_message:</b>      OPTIONAL Message to show the user when he/she has already uploaded a file
 * - <b>upload_to:</b>           OPTIONAL Directory to which to upload the files (default: "/upload")
 * - <b>upload_to_temp:</b>      OPTIONAL Directory to which to upload the temporary files (default: "/upload/temp")
 * - <b>valid_types:</b>         OPTIONAL Valid file MIME types (default: any MIME types allowed)
 * 
 * In order to handle file uploads when a user must re-submit the form
 * due to validation errors, the file name is stored in a hidden <input>
 * field.
 * 
 * @package    Pancake
 * @subpackage Fields
 */
class InputFile extends Input
{
	public static $methodName = 'fileUpload';
	protected $_paragraphClasses = array(
		'field',
		'file-upload'
	);
	protected $_attributes = array(
		'type' => 'file'
	);
	
	// File upload settings
	protected $_validProperties = array(
		'show_upload_message' => true,
		'upload_message' => 'You have already uploaded the file "%s".  Uploading a new file will overwrite this.',
		'upload_to' => null,
		'upload_to_temp' => null,
		'valid_types' => array()
	);
	
	/**
	 * File name without directory
	 * 
	 * @var string
	 */
	protected $_fileName;
	
	/**
	 * Named MIME type groups
	 * 
	 * These groups are useful for easily allowing a "type" of MIME type,
	 * without having to remember a bunch of specific ones.
	 * 
	 * @var array
	 */
	protected $_mimeTypeGroups = array(
		'audio' => array(
			'audio/mpeg'
		),
		'document' => array(
			'application/msword',
			'application/pdf',
			'application/rtf',
			'application/vnd.openxmlformats-officedocument.wordprocessingml.document',
			'application/vnd.ms-office',
			'application/zip',
			'text/plain',
			'text/rtf'
		),
		'images' => array(
			'image/bmp',
			'image/gif',
			'image/jpeg',
			'image/png'
		),
		'video' => array(
			'video/quicktime',
			'video/mpeg'
		)
	);
	
	/**
	 * Field constructor
	 * 
	 * @throws \Pancake\Exception\FileError When the upload directory does not exist
	 * @throws \Pancake\Exception\FileError When the temporary upload directory does not exist
	 */
	protected function _init()
	{
		parent::_init();
		
		// Set a default file upload path
		if (!$this->property('upload_to'))
		{
			$uploadTo = preg_replace('#(.+?)\/(.+?)\.php$#U', '$1', $_SERVER['SCRIPT_FILENAME']) . '/upload';
			$this->property('upload_to', $uploadTo);
		}
		
		// Ensure the upload directory exists
		if (!file_exists($this->property('upload_to')))
		{
			throw new Exception\FileError('The upload directory "' . $this->property('upload_to') . '" does not exist.');
		}
		
		// Set a default temporary file upload path
		if (!$this->property('upload_to_temp'))
		{
			$this->property('upload_to_temp', $this->property('upload_to') . '/temp');
		}
		
		// Ensure the temporary directory exists
		if (!file_exists($this->property('upload_to_temp')))
		{
			throw new Exception\FileError('The temporary upload directory "' . $this->property('upload_to_temp') . '" does not exist.');
		}
		
		// Create the hidden field for storing the file across submissions
		$this->_childFields[$this->name . '_file_name'] = new Input($this->name . '_file_name', 'hidden', array(
			'required' => false
		));
		
		// Add MIME type validation
		if (count($this->property('valid_types')) > 0)
		{
			$validTypes = array();
			foreach ($this->property('valid_types') as $type)
			{
				// Use a named MIME type group
				if (isset($this->_mimeTypeGroups[$type]))
				{
					$validTypes = array_merge($validTypes, $this->_mimeTypeGroups[$type]);
				}
				// Just a plain MIME type
				else
				{
					$validTypes[] = $type;
				}
			}
			
			$this->_addValidation(array('isMIMEType', $validTypes));
		}
	}
	
	/**
	 * Moves the file out of the temporary directory
	 * 
	 * @throws \Pancake\Exception\FileError When the file cannot be moved out of the temporary directory
	 */
	public function moveOutOfTemp()
	{
		if (!empty($this->_fileName))
		{
			if (copy($this->value, $this->property('upload_to') . '/' . $this->_fileName))
			{
				// Delete the temporary file
				unlink($this->value);
				
				// Store the new "permanent" file name
				// FieldInputFile::_value() is not used to avoid calling already performed safety checks
				$this->_value = $this->property('upload_to') . '/' . $this->_fileName;
			}
			else
			{
				throw new Exception\FileError('The file "' . $this->_fileName . '" could not be moved out of the temporary directory.');
			}
		}
	}
	
	/**
	 * Returns both the visible upload field and the hidden field
	 * 
	 * @return string
	 */
	public function output()
	{
		return $this->_childFields[$this->name . '_file_name']->output() . parent::output();
	}
	
	/**
	 * Appends the file-already-uploaded message to the label text
	 * 
	 * @param  string  $text        OPTIONAL Runtime text
	 * @param  boolean $includeTags OPTIONAL Whether to return just the text or a formatted <label> tag
	 * @param  array   $attributes  OPTIONAL <label> tag attributes
	 * @return string
	 */
	public function label($text = null, $includeTags = true, $attributes = array())
	{
		// Only include the message when outputting the <label> tag
		if ($includeTags === true
			&& $this->_fileName != null
			&& $this->property('show_upload_message'))
		{
			// Same as PancakeField::label()
			if ($text == null)
			{
				if ($this->property('label') != null)
				{
					$text = $this->property('label');
				}
				else
				{
					$text = Helper\Readable::process($this->name);
				}
			}
			
			$text .= sprintf(Settings\Fields::UPLOAD_MESSAGE_HTML, sprintf($this->property('upload_message'), preg_replace('#\d+\-(.+?)#', '$1', $this->_fileName)));
		}
		
		return parent::label($text, $includeTags, $attributes);
	}
	
	/**
	 * Moves the uploaded file into the temporary sub-directory
	 * 
	 * Files are immediately moved into a temporary directory, with the
	 * file name being stored in a hidden <input> field in case the user
	 * does not enter valid data.
	 * 
	 * @param  string $value
	 * @return string
	 */
	protected function _value($value = null)
	{
		if ($value !== null)
		{
			// File newly uploaded
			if (!empty($_FILES[$this->name]['name']))
			{
				// Sanitise the user input
				$this->_fileName = '1-' . $this->_makeSafeName($_FILES[$this->name]['name']);
								
				// Try to ensure the file name is unique
				$x = 2;
				while (file_exists($this->property('upload_to') . '/' . $this->_fileName)
					|| file_exists($this->property('upload_to_temp') . '/' . $this->_fileName))
				{
					$this->_fileName = $x . strstr($this->_fileName, '-');
					$x++;
				}
				
				// Move the file to the temporary directory
				$this->_value = $this->property('upload_to_temp') . '/' . $this->_fileName;
				move_uploaded_file($_FILES[$this->name]['tmp_name'], $this->_value);
				
				// Store the file name in the hidden field
				$this->_childFields[$this->name . '_file_name']->value = $this->_fileName;
			}
			// File uploaded, but stored in the hidden field
			elseif ($this->_childFields[$this->name . '_file_name']->value != null)
			{
				// Sanitise the user input
				$tempName = $this->_makeSafeName($this->_childFields[$this->name . '_file_name']->value);
				
				// Ensure the file actually exists
				if (file_exists($this->property('upload_to_temp') . '/' . $tempName))
				{
					$this->_fileName = $tempName;
					$this->_value = $this->property('upload_to_temp') . '/' . $this->_fileName;
				}
			}
		}
		
		return $this->_value;
	}
	
	/**
	 * Returns the file name for the formatted value
	 * 
	 * @return string
	 */
	protected function _formattedValue()
	{
		return $this->_fileName;
	}
	
	/**
	 * Creates a safe file name
	 * 
	 * @param  string $fileName
	 * @return string
	 */
	protected function _makeSafeName($fileName)
	{
		$extension = strrchr($fileName, '.');
		$name = substr($fileName, 0, -strlen($extension));
		
		return Helper\SUA::generateSlug($name) . '.' . Helper\SUA::generateSlug($extension);
	}
}