<?php
/**
 * Pancake Forms
 * 
 * @author  Ethan Poole
 * @package Pancake
 */

namespace Pancake\Field;

/**
 * List of rank-able text <input> fields
 * 
 * Examples:
 * <code>
 * $form->ranking('numbers', array('one', 'two', 'three));
 * $form->ranking('numbers', array(array('one', array('help_text' => 'Uno')), 'two', 'three));
 * $form->ranking('numbers', array('one', 'two', 'three), array('label_element' => 'h4'));
 * </code>
 * 
 * Field-specific properties:
 * - <b>container:</b>     OPTIONAL Tag for the group container (defaults to "<div>")
 * - <b>label_element:</b> OPTIONAL Tag for the group label (defaults to "<h3>")
 * - <b>options:</b>       REQUIRED List options
 * - <b>rank_up_to:</b>    OPTIONAL Number up to which the user must rank the values (defaults to total number of options)
 * 
 * @package    Pancake
 * @subpackage Fields
 * @uses       \Pancake\Field\ListRankingOption
 */
class ListRanking extends FieldList
{
	public static $methodName = 'ranking';
	protected $_paragraphClasses = array(
		'field',
		'list',
		'ranking'
	);
	
	// List settings
	protected $_optionClass = '\Pancake\Field\ListRankingOption';
	protected $_allowMultiples = false;
	
	// Display group in a <div> with a pseudo label
	protected $_validProperties = array(
		'label_element' => 'h3',
		'options' => array(),
		'rank_up_to' => null
	);
	protected $_properties = array(
		'container' => 'div'
	);
	
	/**
	 * Automates the validation
	 */
	protected function _init()
	{
		parent::_init();
		
		// Set the default minimum required ranking values
		if ($this->property('rank_up_to') == null)
		{
			$this->property('rank_up_to', count($this->_childFields));
		}
		
		// Set the validation
		$this->_clearValidation();
		$this->_addValidation(array('IsRanked', $this->property('rank_up_to')));
	}
	
	/**
	 * Encases the output in an unordered list
	 * 
	 * @return string
	 */
	public function output()
	{
		return '<ul>' . parent::output() . '</ul>';
	}
	
	/**
	 * Returns the ranked values of the options
	 * 
	 * @param  mixed $value OPTIONAL Will be ignored
	 * @return array
	 */
	protected function _value($value = null)
	{
		$this->_value = array();
		foreach ($this->childFields as $field)
		{
			$this->_value[] = $field->value;
		}
		return $this->_value;
	}
	
	/**
	 * Returns a comma-separated list of ranked values
	 * 
	 * @return string
	 */
	protected function _formattedValue()
	{
		// Sort the fields by their rank
		$values = array();
		foreach ($this->childFields as $field)
		{
			$values[$field->value] = $field->label(null, false) . ' (' . $field->value . ')';
		}
		ksort($values);
		
		return implode(', ', $values);
	}
}