<?php
/**
 * Pancake Forms
 * 
 * @author  Ethan Poole
 * @package Pancake
 */

namespace Pancake\Helper;

/**
 * Centralised class for all SUA dependencies
 * 
 * @package Pancake
 */
class SUA
{
	/**
	 * Formats a string that *could* be a phone number
	 * 
	 * @author Zachary Johnson
	 * @param  string $number
	 * @return string
	 */
	public static function formatPhoneNumber($number)
	{
		// Strip away any pesky '1-' before the number since US area codes never
		// start with 1 (http://en.wikipedia.org/wiki/List_of_NANP_area_codes)
		$digits = ltrim(preg_replace('/[^0-9]/', '', $number), '1');
		
		$len = strlen($digits);
		
		// Fingers crossed, we hope international numbers are entered with
		// the + before the number and have more than 10 digits...
		if ($len > 10 && $number[0] == '+')
		{
			return $number;
		}
		
		// US with no area code?
		if ($len == 7)
		{
			return substr($digits, 0, 3) . '-' . substr($digits, 3);
		}
		// US with area code?
		elseif ($len == 10)
		{
			return '(' . substr($digits, 0, 3) . ')' . substr($digits, 3, 3) . '-' . substr($digits, 6);
		}
		// Unknown!
		else
		{
			return $number;
		}
	}
	
	/**
	 * Generates a URL-friendly string
	 * 
	 * @author Kamran Ayub
	 * @param  string $phrase
	 * @return string
	 */
	public static function generateSlug($phrase)
	{
		if (is_string($phrase) || is_numeric($phrase))
		{
			$result = strtolower($phrase);
			
			$result = preg_replace('#[^a-z0-9\s-]#', null, $result);
			$result = trim(preg_replace('#[\s-]+#', ' ', $result));
			$result = trim(substr($result, 0, 45));
			$result = preg_replace('#\s#', '-', $result);
			
			return $result;
		}
		else
		{
			trigger_error('$phrase must be a string');
			return null;
		}
	}
	
	/**
	 * Makes a string HTML "safe"
	 * 
	 * @author SUA
	 * @param  string $input
	 * @return string
	 */
	public static function HTMLSafe($string, $allowed_tags = array('<strong>', '</strong>', '<br />', '<em>', '</em>'))
	{
		$matches = array();
		preg_match_all('!&[A-Za-z]+;|&#[0-9]+;|&#x[0-9a-zA-Z]+;|&|<[^>]+>|[^&<]+|[^&]+!', $string, $matches);
		
		foreach ($matches[0] as $id => $substring)
		{
			if ($substring === '&')
			{
				$matches[0][$id] = '&amp;';
			}
			elseif ($substring[0] !== '&'
				&& (!is_array($allowed_tags) || !in_array($substring, $allowed_tags)))
			{
				$matches[0][$id] = htmlspecialchars($matches[0][$id]);
			}
		}
		
		return join('', $matches[0]);
	}
	
	/**
	 * Validates an email address
	 * 
	 * Yes, $$$ZachIs100%Cool!@domain.tld *IS* in fact a valid email address.
	 * 
	 * @author Zachary Johnson
	 * @param  string $email "Name <user@domain.com>" or "user@domain.com"
	 * @return boolean
	 */
	public static function isValidEmail($email)
	{
		if (!is_string($email) && !is_numeric($email))
		{
			trigger_error('$email must be a string.');
			return false;
		}
		
		if (preg_match('/^[^<]*<([^>]+)>$/', $email, $matches))
		{
			$email = $matches[1];
		}
		
		return (preg_match("/^([-^!#$%&'*+\/=?`{|}~\w]+\.)*[-^!#$%&'*+\/=?`{|}~\w]+@(([a-z0-9]+[-]+)*[a-z0-9]+\.)+[a-z0-9]+$/i", $email) > 0);
	}
	
	/**
	 * Validates a US phone number
	 * 
	 * @author Zachary Johnson
	 * @param  string $number
	 * @return boolean
	 */
	public static function isValidUSPhone($number)
	{
		$number = self::formatPhoneNumber($number);
		return (preg_match('!^\([0-9]{3}\)[0-9]{3}-[0-9]{4}$!', $number) > 0);
	}
	
	/**
	 * Connects an array with the word "and" or something
	 * 
	 * @param  SUA
	 * @param  string $separator
	 * @param  array  $parts
	 * @param  string $final_separator OPTIONAL
	 * @return string|null
	 */
	public static function joinAnd($separator, $parts, $final_separator = 'and')
	{
		// Check input types
		if (!is_array($parts) || !is_string($separator))
		{
			trigger_error('$separator must be a string and $parts must be an array.');
			return null;
		}
		elseif (empty($parts))
		{
			return null;
		}
		elseif (count($parts) == 1)
		{
			return array_shift($parts);
		}
		else
		{
			$part = array_pop($parts);
			return join($separator, $parts) . ' ' . $final_separator . ' ' . $part;
		}
	}
	
	/**
	 * Tidies up user-inputed data
	 * 
	 * Tidy it. I didn't say scrub, clean, or sanitise it.
	 * Basically, this function is for taking form input and extracting
	 * actual value sans junk whitespace or PHP addslashes().
	 * 
	 * @author Zachary Johnson
	 * @param  array|string $data
	 * @return array|string|null
	 */
	public static function realInputValue($data)
	{
		if (!is_array($data)
			&& (!is_string($data) && !is_numeric($data)))
		{
			trigger_error('$data must be an array or a string.');
			return null;
		}
		
		// Evil magic quotes...
		if (get_magic_quotes_gpc())
		{
			$data = stripslashes_array($data);
		}
		
		return is_array($data) ? array_map('trim', $data) : trim($data);
	}
	
	/**
	 * Capitalises a title
	 * 
	 * @author Ethan Poole
	 * @param  string $string
	 * @param  string $language OPTIONAL ISO-639 language abbreviation
	 * @return string
	 */
	public static function titleCapitalisation($string, $language = 'en')
	{
		switch ($language)
		{
			// English (oversimplification)
			case 'en':
				// Words in English titles that aren't capitalised
				$dont_capitalise = array(
					// Articles
					'a',
					'an',
					'the',
					
					// Prepositions
					'amid',
					'at',
					'by',
					'down',
					'for',
					'from',
					'in',
					'into',
					'near',
					'of',
					'off',
					'on',
					'onto',
					'out',
					'over',
					'till',
					'to',
					'via',
					'with',
					
					// Conjunctions and subordinators
					'after',
					'although',
					'and',
					'as',
					'because',
					'but',
					'if',
					'lest',
					'nor',
					'or',
					'so',
					'than',
					'that',
					'though',
					'unless',
					'when',
					'whereas',
					'while',
					'whilst',
					'yet',
					
					// To be (excluding the infitive form)
					'is',
					
					// Other
					'else',
					'then',
				);
				
				$words = explode(' ', $string);
				
				foreach ($words as $key => $word)
				{
					// Capitalise initial word and words not in the array
					if ($key == 0 or !in_array($word, $dont_capitalise))
					{
						$words[$key] = ucwords($word);
					}
				}
				
				$string = implode(' ', $words);
			break;
			
			// Swedish (gross oversimplification)
			case 'se':
				$string = ucfirst($string);
			break;
		}
		
		return $string;
	}
}