<?php
/**
 * Pancake Forms
 * 
 * @author  Ethan Poole
 * @package Pancake
 */

namespace Pancake\Hook;

use Pancake\Exception;
use Pancake\Settings;

/**
 * Sends a confirmation email to the user
 * 
 * Examples:
 * <code>
 * $form->addHook('after', 'ConfirmationEmail', array('from' => 'example@example.com'));
 * $form->addHook('after', 'ConfirmationEmail', array('from' => 'example@example.com', 'to' => 'email', 'subject' => 'Homecoming 2010 Parade Registration Received'));
 * </code>
 * 
 * Settings:
 * - <b>from:</b>    From email address
 * - <b>message:</b> OPTIONAL Confirmation message
 * - <b>subject:</b> OPTIONAL Email subject
 * - <b>to:</b>      OPTIONAL To email address (field name or exact value)
 * 
 * This hook sends a confirmation email to whomever submits the form.  Like
 * {@link \Pancake\Hook\EmailForm}, the default settings are designed to be adaptive
 * and will throw exceptions if a value cannot be set.
 * 
 * @package    Pancake
 * @subpackage Hooks
 */
class ConfirmationEmail extends Hook
{
	protected $_validSettings = array(
		'from' => null,
		'message' => null,
		'subject' => null,
		'to' => null
	);
	
	protected $_requiredSettings = array(
		'from'
	);
	
	/**
	 * Sends a confirmation email
	 * 
	 * @param  array $fields
	 * @return boolean          Whether or not the hook ran successfully
	 * 
	 * @throws \Pancake\Exception\Exception When a "from" email is not specified
	 * @throws \Pancake\Exception\Exception When a "to" email is not specified
	 * @throws \Pancake\Exception\Exception When the confirmation email is not emailed
	 */
	public function run($fields)
	{
		// Bring settings into the local scope
		$from = $this->_settings['from'];
		$message = $this->_settings['message'];
		$subject = $this->_settings['subject'];
		$to = $this->_settings['to'];
		
		// Default subject
		if ($subject == null)
		{
			$subject = Settings\Hooks::CONFIRMATION_SUBJECT;
		}
		
		// To email
		// By default look for an "email" form field value
		if ($to == null && isset($fields['email']))
		{
			$to = $fields['email']->value;
		}
		// You can also pass the name of a form field
		elseif (isset($fields[$to]))
		{
			$to = $fields[$to]->value;
		}
		elseif ($to == null)
		{
			throw new Exception\Exception('No email address could be extracted from the form for the "to" field.');
			return false;
		}
		
		// Email body message
		if ($message == null)
		{
			$message = Settings\Hooks::CONFIRMATION_MESSAGE;
		}
		
		// NOTE: $from has already been validated, so no injection can take place here...
		if (mail($to, $subject, $message, 'From: ' . $from))
		{
			return true;
		}
		else
		{
			throw new Exception\Exception('The confirmation email was not successfully emailed.');
			return false;
		}
	}
}