<?php
/**
 * Pancake Forms
 * 
 * @author  Ethan Poole
 * @package Pancake
 */

namespace Pancake\Hook;

use Pancake\Exception;

/**
 * Emails the form data
 * 
 * Examples:
 * <code>
 * $form->addHook('after', 'EmailForm', array('to' => 'custom@umn.edu.'));
 * $form->addHook('after', 'EmailForm', array('to' => 'custom@umn.edu.', 'from' => 'email'));
 * $form->addHook('after', 'EmailForm', array('exclude_fields' => array('submit', 'private')));
 * </code>
 * 
 * Settings:
 * - <b>exclude_fields:</b>   OPTIONAL An array of fields to exclude from the email
 * - <b>from:</b>             OPTIONAL From email address (field name or exact value)
 * - <b>from_name:</b>        OPTIONAL From name (field name or exact value)
 * - <b>message_template:</b> OPTIONAL Template from which to create the email body (file path or string)
 * - <b>subject:</b>          OPTIONAL Email subject
 * - <b>to:</b>               To email address
 * 
 * The default settings are designed to be adaptive, but the hook will throw
 * an exception if a value cannot be found for a setting.
 * 
 * If the message template is a string, the hook will replace {{ body }} with
 * the automatically generated body, allowing you to specify a greeting and
 * salutation without creating a template file.
 * 
 * @package    Pancake
 * @subpackage Hooks
 */
class EmailForm extends Hook
{
	protected $_validSettings = array(
		'exclude_fields' => array(),
		'from' => null,
		'from_name' => null,
		'message_template' => null,
		'subject' => null,
		'to' => null
	);
	
	protected $_requiredSettings = array(
		'to'
	);
	
	/**
	 * Emails the form data
	 * 
	 * @param  array $fields
	 * @return boolean          Whether or not the hook ran successfully
	 * 
	 * @throws \Pancake\Exception\Exception When a "to" email is not specified
	 * @throws \Pancake\Exception\Exception When a "from" email is not specified
	 * @throws \Pancake\Exception\Exception When the form data is not emailed
	 */
	public function run($fields)
	{
		// Bring settings into the local scope
		$from = $this->_settings['from'];
		$fromName = $this->_settings['from_name'];
		$messageTemplate = $this->_settings['message_template'];
		$subject = $this->_settings['subject'];
		$to = $this->_settings['to'];
		
		// Default subject
		if ($subject == null)
		{
			$subject = 'Web Form Submitted from ' . $_SERVER['SERVER_NAME'] . $_SERVER['REQUEST_URI'];
		}
		
		// From email
		// By default look for an "email" form field value
		if ($from == null && isset($fields['email']))
		{
			$from = $fields['email']->value;
		}
		// You can also pass the name of a form field
		elseif (isset($fields[$from]))
		{
			$from = $fields[$from]->value;
		}
		elseif ($from == null)
		{
			throw new Exception\Exception('No email was designated for the "from" field.');
			return false;
		}
		
		// From name (optional)
		if ($fromName == null)
		{
			// Look for a "name" form field value
			if (isset($fields['name']))
			{
				$fromName = $fields['name']->value;
			}
			// Look for "first name" and "last name" form field values
			elseif (isset($fields['first_name']) && isset($fields['last_name']))
			{
				$fromName = $fields['first_name']->value . ' ' . $fields['last_name']->value;
			}
			// Look for a "first name" form field value
			elseif (isset($fields['first_name']))
			{
				$fromName = $fields['first_name']->value;
			}
		}
		// You can pass the name of a form field
		elseif (isset($fields[$fromName]))
		{
			$fromName = $fields[$fromName]->value;
		}
		
		// Compute full from field to pass to sendemail()
		if ($fromName != null)
		{
			$from = $fromName . ' <' . $from . '>';
		}
		
		/* EMAIL BODY */
			$body = null;
			
			// Email template file
			if (file_exists($messageTemplate))
			{
				ob_start();
				include $messageTemplate;
				$body = ob_get_clean();
			}
			else
			{
				foreach ($fields as $field)
				{
					if ($field->formattedValue != null
						&& !in_array($field->name, $this->_settings['exclude_fields']))
					{
						// Use a short label if possible
						if ($field->property('label_short') != null)
						{
							$label = $field->property('label_short');
						}
						// Otherwise, use the normal label
						else
						{
							$label = $field->label(null, false);
						}
						
						$body .= '==' . $label . '==' . "\n" . $field->formattedValue . "\n\n";
					}
				}
				
				// Insert body into the message template (string)
				if ($messageTemplate != null)
				{
					$body = str_replace('{{ body }}', $body, $messageTemplate);
				}
			}
		/* END EMAIL BODY */
		
		// NOTE: $from has already been validated, so no injection can take place here...
		if (mail($to, $subject, $body, 'From: ' . $from))
		{
			return true;
		}
		else
		{
			throw new Exception\Exception('The form data was not successfully emailed.');
			return false;
		}
	}
}