<?php
/**
 * Pancake Forms
 * 
 * @author  Ethan Poole
 * @package Pancake
 */

namespace Pancake\Hook;

use Pancake\Exception;

/**
 * Base hook class
 * 
 * Hooks are used to execute pre-display and post-submit logic, such as emailing the
 * form, sending a confirmation email, or hiding the form after a specified date.
 * 
 * @package    Pancake
 * @subpackage Hooks
 */
abstract class Hook
{
	/**
	 * Hook settings
	 * 
	 * @var array
	 */
	protected $_settings = array();
	
	/**
	 * Valid settings
	 * 
	 * @var array
	 */
	protected $_validSettings = array();
	
	/**
	 * Required settings
	 * 
	 * @var array
	 */
	protected $_requiredSettings = array();
	
	/**
	 * Configures hook settings
	 * 
	 * @param  array $settings              OPTIONAL Hook settings
	 * @throws \Pancake\Exception\Exception When not all of the settings are valid
	 * @throws \Pancake\Exception\Exception When a required setting is missing
	 */
	public function __construct(array $settings = array())
	{
		// Ensure all settings are valid
		if (!empty($this->_validSettings)
			&& count(array_intersect_key($this->_validSettings, $settings)) != count($settings))
		{
			throw new Exception\Exception('Not all of the settings are valid: ' . implode(', ', array_keys($settings)));
		}
		
		// Ensure all required settings are set
		foreach ($this->_requiredSettings as $setting)
		{
			if (empty($settings[$setting]))
			{
				throw new Exception\Exception('The setting "' . $setting . '" is required.');
			}
		}
		
		$this->_settings = array_merge($this->_validSettings, $settings);
		$this->_init();
	}
	
	/**
	 * Hook specific constructor
	 * 
	 * Called after {@link \Pancake\Hook\Hook::__construct()}.
	 */
	protected function _init()
	{
		
	}
	
	/**
	 * Hook logic
	 * 
	 * @param  array   $fields Form fields
	 * @return boolean|string  True if all is successful or false/error message otherwise
	 */
	public function run($fields)
	{
		return true;
	}
}