<?php
/**
 * Pancake Forms
 * 
 * @author  Ethan Poole
 * @package Pancake
 */

namespace Pancake\Loader;

use Pancake\Exception;

/**
 * Field loader
 * 
 * @package Pancake
 */
class FieldLoader implements Loader
{
	/**
	 * Directories from which to load field classes
	 * 
	 * Format:
	 * 'namespace' => 'directory'
	 * 
	 * @var array
	 */
	protected $_directories = array();
	
	/**
	 * Loaded fields
	 * 
	 * Format:
	 * 'name' => instance
	 * 
	 * @var array
	 */
	protected $_fields = array();
	
	/**
	 * File and directory names to ignore
	 * 
	 * @var array
	 */
	protected $_ignore = array(
		'.',
		'..'
	);
	
	/**
	 * Valid fields that can be used
	 * 
	 * Format:
	 * 'method name' => 'namespaced class name'
	 * 
	 * @var array
	 */
	protected $_validFields = array();
	
	/**
	 * Initiates the class instance
	 * 
	 * @param array   $directories       Directories from which to load field classes
	 * @param boolean $automaticallyLoad Whether to load the fields automatically
	 */
	public function __construct($directories, $automaticallyLoad = true)
	{
		$this->_directories = $directories;
		
		// Load the fields automatically
		if ($automaticallyLoad)
		{
			$this->loadAll();
		}
	}
	
	/**
	 * @return integer
	 */
	public function count()
	{
		return count($this->_fields);
	}
	
	/**
	 * @return array
	 */
	public function getIterator()
	{
		return new \ArrayIterator($this->_fields);
	}
	
	/**
	 * Loads a specific field class
	 * 
	 * @param  string $fieldType            Method or class name to load
	 * @param  array  $parameters           Parameters with which to create the class
	 * @return mixed                        Field class instance
	 * 
	 * @throws \Pancake\Exception\NameError When the field does not exist
	 * @throws \Pancake\Exception\NameError When duplicate field names occur
	 */
	public function &load($fieldType, $parameters = array())
	{
		// Load from a method name
		if (isset($this->_validFields[$fieldType]))
		{
			$field = new $this->_validFields[$fieldType]($parameters);
		}
		// Load from a namespaced class name
		elseif (in_array($fieldType, $this->_validFields))
		{
			$field = new $fieldType($parameters);
		}
		else
		{
			throw new Exception\NameError('The field "' . $fieldType . '" does not exist.');
			return false;
		}
		
		// Ensure the field name is not already being used
		if (in_array($field->name, array_keys($this->_fields)))
		{
			throw new Exception\NameError('The field name "' . $field->name . '" is already being used.');
			return false;
		}
		
		// Save the field class instance
		$this->_fields[$field->name] = &$field;
		
		return $field;
	}
	
	/**
	 * Inventories the available field classes
	 * 
	 * @return boolean Whether or not the fields were newly loaded
	 * 
	 * @throws \Pancake\Exception\FileError When a directory cannot be opened
	 * @throws \Pancake\Exception\NameError When a file does not have a corresponding class
	 * @throws \Pancake\Exception\NameError When a method name is already being used
	 */
	public function loadAll()
	{
		// Ensure that this method has not already run
		if (!empty($this->_validFields))
		{
			return false;
		}
		
		// Loop through each field directory and load the classes
		foreach ($this->_directories as $namespace => $directory)
		{
			if ($handle = @opendir($directory))
			{
				while (($file = readdir($handle)) !== false)
				{
					if (!in_array($file, $this->_ignore))
					{
						// Remove the file extension
						$file = substr($file, 0, strrpos($file, '.')); 
						$className = $namespace . $file;
						
						// Make sure that a corresponding class exists
						if (class_exists($className))
						{
							// Make sure that the class has a method name
							if (property_exists($className, 'methodName'))
							{
								// Get the method name
								$methodName = get_class_vars($className);
								$methodName = $methodName['methodName'];
								
								// Allow user creation to be disabled
								if (!empty($methodName))
								{
									// Make sure the method name is not already being used
									if (!isset($this->_validFields[$methodName]))
									{
										$this->_validFields[$methodName] = $className;
									}
									else
									{
										throw new Exception\NameError('The method name "' . $methodName . '" is already being used and cannot be used by ' . $className . '.');
									}
								}
							}
						}
						else
						{
							throw new Exception\NameError('The file "' . $file . '.php" does not have a corresponding class.');
						}
					}
				}
				
				closedir($handle);
			}
			else
			{
		 		throw new Exception\FileError('The directory "' . $directory . '" could not be opened.');
			}
		}
		
		return true;
	}
	
	/**
	 * @param  string $offset
	 * @param  mixed  $value
	 * @throws \Pancake\Exception\Exception Whenever called in order to disable runtime changes
	 */
	public function offsetSet($offset, $value)
	{
		throw new Exception\Exception('Modification of fields not allowed via the ArrayAccess interface.');
	}
	
	/**
	 * @param  string $offset
	 * @return boolean
	 */
	public function offsetExists($offset)
	{
		return isset($this->_fields[$offset]);
	}
	
	/**
	 * @param string $offset
	 */
	public function offsetUnset($offset)
	{
		unset($this->_fields[$offset]);
	}
	
	/**
	 * @param  string $offset
	 * @return mixed
	 */
	public function offsetGet($offset)
	{
		return isset($this->_fields[$offset]) ? $this->_fields[$offset] : null;
	}
	
	/**
	 * @param  string $member
	 * @return mixed
	 */
	public function __get($member)
	{
		if ($member == 'validFields')
		{
			return $this->_validFields;
		}
	}
}