<?php
/**
 * Pancake Forms
 * 
 * @author  Ethan Poole
 * @package Pancake
 */

namespace Pancake;

/**
 * @see \Pancake\Autoloader
 */
require_once 'Pancake/Autoloader.php';

/**
 * Pancake form base class
 * 
 * This base form class handles creating form fields, processing submitted
 * forms, and running post-submit hooks.  It includes a few static methods
 * as helpers for the Pancake library as a whole.
 * 
 * @package       Pancake
 * @property-read string  $errors
 * @property-read string  $resources
 */
abstract class PancakeAbstract
{
	/**
	 * Form settings
	 * 
	 * Settings:
	 * - <b>directory_fields:</b> Directory of field classes
	 * - <b>identifier:</b>       Form identifier (useful for pages with multiple forms)
	 * - <b>success:</b>          Success message
	 * 
	 * Get and set through {@link \Pancake\PancakeAbstract::setting()}.
	 * 
	 * @var array
	 * @see Pancake::setting()
	 */
	protected $_settings = array(
		'directions' => null,
		'directory_fields' => 'Field',
		'identifier' => null,
		'success' => '<p>Your form was sent successfully!  Student Unions & Activities thanks you very much for your submission!</p>'
	);
	
	/**
	 * Field loader
	 * 
	 * Add fields through the magic methods and {@link \Pancake\PancakeAbstract::__call()}.
	 * Get fields through the field name and {@link \Pancake\PancakeAbstract::__get()}.
	 * 
	 * @var \Pancake\Loader\FieldLoader
	 */
	protected $_fields;
	
	/**
	 * Map of conditional fields
	 * 
	 * Conditional fields are fields that are required only if another
	 * field has a specific value.  This array holds the field name and
	 * the conditional field name and value.
	 * 
	 * @var array
	 */
	protected $_conditionalFields = array();
	
	/**
	 * Form errors
	 * 
	 * Get errors through {@link \Pancake\PancakeAbstract::errors()}.
	 * 
	 * @var array
	 * @see Pancake::errors()
	 */
	protected $_errors = array();
	
	/**
	 * Form hooks
	 * 
	 * Add hooks through {@link \Pancake\PancakeAbstract::addHook()}.  "Before" hooks process
	 * before the form is shown, allowing them to stop the form from being
	 * displayed.  "After" hooks process after the form is submitted and
	 * validated, allowing them to dispatch emails, modify a database, etc.
	 * 
	 * If "before" hooks return a string, the string is displayed instead of
	 * the form.  "After" hooks should only return a boolean.
	 * 
	 * @var array
	 * @see Pancake::addHook()
	 */
	protected $_hooks = array(
		'before' => array(),
		'after' => array()
	);
	
	/**
	 * Preset values
	 * 
	 * Add a preset value through {@link \Pancake\PancakeAbstract::presetValue()}.
	 * 
	 * @var array
	 * @see Pancake::presetValue()
	 */
	protected $_presetValues = array();
	
	/**
	 * Notifies the webmaster of a runtime (hook) error
	 * 
	 * @param  string $subject
	 * @param  string $message
	 * @return boolean
	 */
	abstract protected function _notifyWebmaster($subject, $message);
	
	/**
	 * Configures form settings and loads valid form fields
	 * 
	 * @param array $settings Runtime settings
	 */
	public function __construct($settings = array())
	{
		// Allow settings to be overriden at initialisation
		$this->_settings = array_merge($this->_settings, $settings);
		
		// Load valid form fields
		$this->_fields = new Loader\FieldLoader(array(
			'\Pancake\Field\\' => PANCAKE_ROOT . $this->setting('directory_fields')
		));
	}
	
	/**
	 * Starts the form
	 * 
	 * This function returns the opening <form> tag, but also starts a buffer
	 * that should be cleaned and flushed using {@link \Pancake\PancakeAbstract::end()}.
	 * 
	 * @param  array  $attributes OPTIONAL Attributes for <form>
	 * @return string             Opening <form> tag
	 */
	public function start($attributes = array())
	{
		$defaultAttributes = array(
			'action' => $_SERVER['REQUEST_URI'],
			'class' => 'pancake',
			'method' => 'post'
		);
		$validAttributes = array('accept', 'accept-charset', 'action', 'class', 'enctype', 'id', 'method', 'title');
		
		ob_start();
		return '<form' . Helper\Attributes::process($attributes, $defaultAttributes, $validAttributes) . '>' . $this->_setSpamProtections();
	}
	
	/**
	 * Ends the form
	 * 
	 * This function does the following (with sub-lists denoting alternative paths):
	 * <ol>
	 *     <li>Cleans and flushes the buffer started by {@link \Pancake\PancakeAbstract::start()}.</li>
	 *     <li>
	 *         Determines if the form has been submitted:
	 *         <ol>
	 *             <li>Assigns the fields with values.</li>
	 *             <li>
	 *                 Validates the fields:
	 *                 <ol>
	 *                     <li>Runs any form hooks.</li>
	 *                     <li>Displays a success message.</li>
	 *                 </ol>
	 *             </li>
	 *             <li>Shows form errors.</li>
	 *         </ol>
	 *     </li>
	 *     <li>Displays the processed form.</li>
	 * </ol>
	 * 
	 * @return string
	 */
	public function end()
	{
		$buffer = ob_get_clean() . '</form>';
		$identifier = $this->setting('identifier');
		
		// Run before hooks
		$beforeHooksReturn = $this->_runHooks('before');
		if ($beforeHooksReturn !== true)
		{
			return $beforeHooksReturn;
		}
		
		// Form submitted?
		if ((!empty($identifier) && !empty($_POST[$identifier])) // Via identifier
			|| (empty($identifier) && !empty($_POST))) // Assume the only form on the page
		{
			// Perform a spam protection check
			$spamCheck = $this->_checkSpamProtections();
			if ($spamCheck !== true)
			{
				$this->_errors[] = array(Settings\Security::SPAM_MESSAGE);
			}
			
			// Record the fields that trigger conditional fields
			$conditionalFields = array();
			foreach ($this->_conditionalFields as $condition)
			{
				$conditionalFields[] = $condition[1];
			}
			$conditionalFields = array_unique($conditionalFields);
			
			// Assign fields with values
			foreach ($this->_fields as $field)
			{
				// Child fields
				if (count($field->childFields) > 0)
				{
					foreach ($field->childFields as $child)
					{
						if (array_key_exists($child->name, $_POST))
						{
							$child->value = Helper\SUA::realInputValue($_POST[$child->name]);
						}
					}
				}
				
				// Single field
				if (array_key_exists($field->name, $_POST))
				{
					$field->value = Helper\SUA::realInputValue($_POST[$field->name]);
				}
				// Make sure to pass an empty value if the POST member is empty (for any default values)
				else
				{
					$field->defaultValue();
				}
				
				// See if any fields rely on this field's value as a conditional requirement
				if (in_array($field->name, $conditionalFields))
				{
					$value = is_array($field->value) ? $field->value[0] : $field->value;
					
					foreach ($this->_conditionalFields as $condition)
					{
						if ($condition[1] == $field->name
							&& $condition[2] == $value)
						{
							$this->_fields[$condition[0]]->property('required', true);
						}
					}
				}
			}
			
			// Proceed processing form if the data validates
			if ($this->isValid())
			{
				// Run after hooks
				if ($this->_runHooks('after') === true)
				{
					$buffer = sprintf(Settings\Validation::SUCCESS_HTML, $this->setting('success'));
				}
				// Show an error message if the hooks fail
				else
				{
					$buffer = sprintf(Settings\Validation::ERROR_ITEM_HTML, 'A technical error occurred with your submission.  The webmaster has been notified of the error.  Please try to submit the form again later and we apologise for the inconvenience.');
					$buffer = sprintf(Settings\Validation::ERROR_HTML, $buffer);
				}
			}
			// Otherwise, display the errors
			else
			{
				$buffer = preg_replace('#<form(.+?)>#', '<form$1>' . str_replace('$', '\$', $this->errors), $buffer);
			}
		}
		// If not submitted, insert the form directions
		elseif ($this->setting('directions') !== null)
		{
			$buffer = preg_replace('#<form(.+?)>#', str_replace('$', '\$', $this->setting('directions')) . '<form$1>', $buffer);
		}
		
		$buffer = preg_replace('#{{ Field: (.+?) }}#e', '$this->_fields["$1"]->__toString()', $buffer);
		return $buffer;
	}
	
	/**
	 * Presets a value for a field
	 * 
	 * Preset field values are useful for combining add and edit forms, amongst
	 * other things.  {@link \Pancake\PancakeAbstract::addField()} will search for
	 * any preset values and pass it to the newly created form field.
	 * 
	 * @param string $fieldName 
	 * @param mixed  $value
	 */
	public function presetValue($fieldName, $value)
	{
		$this->_presetValues[$fieldName] = $value;
	}
	
	/**
	 * Adds a field to the form
	 * 
	 * This method should be used in conjunction with the field magic
	 * method names and {@link \Pancake\PancakeAbstract::__call()}.
	 * 
	 * @param  string $fieldClass Field class name
	 * @param  array  $parameters Field parameters (e.g. names, attributes, etc.)
	 * @return string             Marker to the field that will be replaced in {@link \Pancake\PancakeAbstract::end()}
	 */
	public function addField($fieldClass, $parameters = array())
	{
		// Attempt to create an instance of the field class
		$field = $this->_fields->load($fieldClass, $parameters);
		
		// Determine if the field can be used as a form identifier
		if ($field->canBeFormIdentifier)
		{
			$this->setting('identifier', $field->name);
		}
		
		// Record any conditional requirement settings
		if ($field->property('required_if') !== null
			&& is_array($field->property('required_if')))
		{
			$condition = $field->property('required_if');
			$this->_conditionalFields[] = array($field->name, $condition[0], $condition[1]);
		}
		
		// Assign any preset value to the field
		if (isset($this->_presetValues[$field->name]))
		{
			$field->value = $this->_presetValues[$field->name];
		}
		
		return '{{ Field: ' . $field->name . ' }}';
	}
	
	/**
	 * Adds a form hook
	 * 
	 * @param  string $type     Hook type, "before" or "after"
	 * @param  string $hook     Hook name
	 * @param  array  $settings OPTIONAL Hook settings
	 * @return boolean          Whether or not the hook was added successfully
	 * 
	 * @throws \Pancake\Exception\Exception    When the hook type is not valid
	 * @throws \Pancake\Exception\DoesNotExist When the hook class does not exist
	 */
	public function addHook($type, $hook, array $settings = array())
	{
		// Make sure the hook type is valid
		if (!array_key_exists($type, $this->_hooks))
		{
			throw new Exception\Exception('The form hook type "' . $type . '" is not a valid hook type.');
			return false;
		}
		
		// Included hook
		if (file_exists(PANCAKE_ROOT . 'Hook/' . $hook . '.php'))
		{
			$className = '\Pancake\\Hook\\' . $hook;
			$this->_hooks[$type][] = new $className($settings);
			
			return true;
		}
		// User-defined hook
		elseif (class_exists($hook))
		{
			$this->_hooks[$type][] = new $hook($settings);
			return true;
		}
		else
		{
			throw new Exception\DoesNotExist('The form hook "' . $hook . '" does not exist.');
			return false;
		}
	}
	
	/**
	 * Determines whether or not the form field is valid
	 * 
	 * @return boolean
	 */
	public function isValid()
	{
		foreach ($this->_fields as $key => $field)
		{
			if (!$field->isValid())
			{
				$this->_errors[$key] = $field->errors;
			}
		}
		
		return empty($this->_errors);
	}
	
	/**
	 * Returns the form errors
	 * 
	 * @param  boolean $format   OPTIONAL Whether to return a formatted list or an array of errors
	 * @return string|array|null Formatted errors or an array of raw error data
	 */
	public function errors($format = true)
	{
		if (empty($this->_errors))
		{
			return null;
		}
		
		// Return array of errors
		if ($format === false)
		{
			$formatedErrors = array();
			
			foreach ($this->_errors as $fieldKey => $errors)
			{
				// For field-based errors, make sure the errors are set to show
				if (isset($this->_fields[$fieldKey])
					&& $this->_fields[$fieldKey]->property('show_errors') === true)
				{
					$formatedErrors[$this->_fields[$fieldKey]->name] = $errors;
				}
				// Just add non-field-based errors
				elseif (!isset($this->_fields[$fieldKey]))
				{
					$formatedErrors[] = $errors;
				}
			}
			
			return $formatedErrors;
		}
		// Return a formatted error display
		else
		{
			$formatedErrors = array();
			
			foreach ($this->_errors as $fieldKey => $errors)
			{
				// For field-based errors, make sure the errors are set to show
				if (isset($this->_fields[$fieldKey])
					&& $this->_fields[$fieldKey]->property('show_errors') === true)
				{
					// Use a short label if possible
					if ($this->_fields[$fieldKey]->property('label_short') != null)
					{
						$label = $this->_fields[$fieldKey]->property('label_short');
					}
					// Otherwise, use the normal label
					else
					{
						$label = $this->_fields[$fieldKey]->label(null, false);
					}
					
					foreach ($errors as $error)
					{
						$formatedErrors[] = sprintf(Settings\Validation::ERROR_ITEM_FIELD_HTML, $label, $error);
					}
				}
				// Just add non-field-based errors
				elseif (!isset($this->_fields[$fieldKey]))
				{
					foreach ($errors as $error)
					{
						$formatedErrors[] = sprintf(Settings\Validation::ERROR_ITEM_HTML, $error);
					}
				}
			}
			
			if (!empty($formatedErrors))
			{
				return sprintf(Settings\Validation::ERROR_HTML, implode('', $formatedErrors));
			}
			else
			{
				return null;
			}
		}
	}
	
	/**
	 * Gets and sets a form setting
	 * 
	 * @param  string $key                     Setting name
	 * @param  string $value                   OPTIONAL Value to assign the setting
	 * @return string|boolean
	 * @throws \Pancake\Exception\DoesNotExist When the settings key does not exist
	 */
	public function setting($key, $value = null)
	{
		if (!array_key_exists($key, $this->_settings))
		{
			throw new Exception\DoesNotExist('The setting "' . $key . '" does not exist.');
			return false;
		}
		else
		{
			if ($value != null)
			{
				$this->_settings[$key] =  (string) $value;
			}
			
			return $this->_settings[$key];
		}
	}
	
	/**
	 * Sets the CSRF token and creates the honeypot field
	 * 
	 * <b>CSRF protection:</b>
	 * To protect against cross-site request forgery, this method sets a token
	 * in the session and then saves this value as a hidden <input> in the form.
	 * Then {@link \Pancake\PancakeAbstract::_checkSpamProtections()} ensures the session value and
	 * the <input> value match exactly, hence also ensuring the form is submitted
	 * from the intended page and not a third-party website.
	 * 
	 * <b>Honeypot protection:</b>
	 * To protect against spam bots, this method sets a field hidden to users.
	 * Most spam bots attempt to inject a value into every form field to avoid
	 * validation issues, so any value inserted into this field will flag the
	 * form as spam.  The <label> text is written to warn any non-visual users
	 * against writing something in it.  The <input> cannot be of the "type"
	 * hidden, since most spam bots ignore these.
	 * 
	 * @return string Spam protection fields
	 */
	protected function _setSpamProtections()
	{
		// Set the CSRF session token
		if (!isset($_SESSION['pancake']['csrf_token']))
		{
			$_SESSION['pancake']['csrf_token'] = md5(Settings\Security::SALT . time());
		}
		
		return '<div class="hide">
			<input name="csrf_token" type="hidden" value="' . $_SESSION['pancake']['csrf_token'] . '" />
			<p>
				<label for="f_honeypot">If you enter anything in this field your comment will be treated as spam:</label>
				<input id="f_honeypot" name="honeypot" type="text" />
			</p>
		</div>';
	}
	
	/**
	 * Checks the CSRF token and the honeypot field
	 * 
	 * See {@link \Pancake\PancakeAbstract::_setSpamProtections()} for a description
	 * of Pancake's internal spam protection techniques.
	 * 
	 * @return boolean Whether or not the form is spam
	 */
	protected function _checkSpamProtections()
	{
		$hasCSRFtoken = false;
		$honeypotIsEmpty = false;
		
		// Match the CSRF token in the session and the post data
		if (isset($_POST['csrf_token'])
			&& isset($_SESSION['pancake']['csrf_token'])
			&& $_POST['csrf_token'] == $_SESSION['pancake']['csrf_token'])
		{
			$hasCSRFtoken = true;
		}
		
		// Check that the honeypot <input> is empty
		if (isset($_POST['honeypot'])
			&& $_POST['honeypot'] == '')
		{
			$honeypotIsEmpty = true;
		}
		
		return $hasCSRFtoken && $honeypotIsEmpty;
	}
	
	/**
	 * Runs a series of form hooks
	 * 
	 * @param  string $type                    Hook type, "before" or "after"
	 * @return boolean|string                  True for success or a string/false for an error
	 * @throws \Pancake\Exception\DoesNotExist When the hook type is not valid
	 */
	protected function _runHooks($type)
	{
		// Make sure the hook type is valid
		if (!array_key_exists($type, $this->_hooks))
		{
			throw new Exception\DoesNotExist('The form hook type "' . $type . '" is not a valid hook type.');
			return false;
		}
		
		$return = true;
		$hookErrors = array();
		
		// Run each hook, but only save the error from the top-most failure
		foreach ($this->_hooks[$type] as $hook)
		{
			try
			{
				$hookReturn = $hook->run($this->_fields);
				
				if ($hookReturn !== true && $return === true)
				{
					$return = $hookReturn;
				}
			}
			catch (PancakeException $e)
			{
				$hookErrors[] = $e->getMessage();
				$return = false;
			}
		}
		
		// Notify the webmaster about any exceptions thrown
		if (!empty($hookErrors))
		{
			$body = "Form hook exceptions were thrown on " . $_SERVER['SERVER_NAME'] .  $_SERVER['REQUEST_URI'] . " and probably should be investigated:\n\n" . implode("\n", $hookErrors);
			$this->_notifyWebmaster('Form Error on ' . $_SERVER['SERVER_NAME'] . $_SERVER['REQUEST_URI'], $body);
		}
		
		return $return;
	}
	
	/**
	 * Magic getters
	 * 
	 * @param  string $member
	 * @return mixed
	 */
	public function __get($member)
	{
		switch ($member)
		{
			case 'errors':
				return $this->errors();
			break;
			
			case 'resources':
				return $this->resources();
			break;
		}
		
		// Return field if possible
		if (isset($this->_fields[$member]))
		{
			return $this->_fields[$member];
		}
	}
	
	/**
	 * Adds a form field via a magic method
	 * 
	 * @param  string $name       Field name
	 * @param  array  $parameters OPTIONAL Field parameters
	 * @return string
	 */
	public function __call($name, $parameters)
	{
		try
		{
			return $this->addField($name, $parameters);
		}
		catch (Exception\NameError $e)
		{
			return $e->getMessage();
		}
	}
}