<?php
/**
 * Pancake Forms
 * 
 * @author     Ethan Poole
 * @package    Pancake
 * @subpackage Validators
 */

namespace Pancake\Validator;

/**
 * File size validator
 * 
 * Examples:
 * <code>
 * $fileSize->validate('small_file', 1000); // True
 * $fileSize->validate('big_file', 1000); // False
 * </code>
 * 
 * @package    Pancake
 * @subpackage Validators
 */
class HasFileSize extends Validator
{
	/**
	 * Dynamic error message template
	 */
	const ERROR_MESSAGE = 'Please upload a file smaller than %s.';
	
	/**
	 * @param  mixed   $value
	 * @param  integer $allowedSize Allowed file size in bytes
	 * @return boolean
	 */
	protected function _isTrue($value, $allowedSize)
	{
		// Set the dynamic error message
		$this->_error = $this->_formulateErrorMessage($allowedSize);
		
		// Make sure the file exists
		if (!file_exists($value))
		{
			return false;
		}
		
		return filesize($value) <= $allowedSize;
	}
	
	/**
	 * Displays the most human-readable file-size format
	 * 
	 * @param  integer $allowedSize Allowed file size in bytes
	 * @return string               Error message
	 */
	protected function _formulateErrorMessage($allowedSize)
	{
		// Gigabytes
		if ($allowedSize > pow(2, 30))
		{
			$allowedSize .= ' GB';
		}
		// Megabytes
		elseif ($allowedSize > pow(2, 20))
		{
			$allowedSize .= ' MB';
		}
		// Kilobytes
		elseif ($allowedSize >  pow(2, 10))
		{
			$allowedSize .= ' kB';
		}
		// Bytes
		else
		{
			$allowedSize .= ' bytes';
		}
		
		return sprintf(hasFileSize::ERROR_MESSAGE, $allowedSize);
	}
}