<?php
/**
 * Pancake Forms
 * 
 * @author     Ethan Poole
 * @package    Pancake
 * @subpackage Validators
 */

namespace Pancake\Validator;

/**
 * Word count validator
 * 
 * Examples:
 * <code>
 * $hasWordCount->validate('string', 5); // Minimum of five words; False
 * $hasWordCount->validate('string', 1, 10); // Between one and ten words; True
 * </code>
 * 
 * @package    Pancake
 * @subpackage Validators
 * @author     Thanks to {@link http://us2.php.net/manual/en/function.str-word-count.php#79699}
 */
class HasWordCount extends Validator
{
	/**
	 * Error message for a minimum word count
	 */
	const MIN_MESSAGE = 'Please enter at least %s words.';
	
	/**
	 * Error message for a maximum word count
	 */
	const MAX_MESSAGE = 'Please enter no more than %s words.';
	
	/**
	 * Error message for a limited-range word count
	 */
	const RANGE_MESSAGE = 'Please enter between %s and %s words.';
	
	/**
	 * @param  mixed   $value
	 * @param  integer $min   Minimum number of words required
	 * @param  integer $max   OPTIONAL Maximum number of words allowed
	 * @return boolean
	 */
	protected function _isTrue($value, $min, $max = null)
	{
		// Set the dynamic error message
		$this->_error = $this->_formulateErrorMessage($min, $max);
		
		/* DETERMINE WORD COUNT */
			// Strip all HTML tags
			$wordCount = strip_tags($value);
			
			// Remove "words" that don't consist of alphanumerical characters or punctuation
			$wordCount = trim(preg_replace('#[^(\w|\d|\'|\"|\.|\!|\?|;|,|\\|\/|\-|:|\&|@)]+#', ' ', $wordCount));
			
			// Remove one-letter "words" that consist only of punctuation
			$wordCount = trim(preg_replace('#\s*[(\'|\"|\.|\!|\?|;|,|\\|\/|\-|:|\&|@)]\s*#', ' ', $wordCount));
			
			// Remove superfluous whitespace
			$wordCount = preg_replace('#\s\s+#', ' ', $wordCount);
			
			$wordCount = explode(' ', $wordCount);
			$wordCount = array_filter($wordCount);
			$wordCount = count($wordCount);
		/* END DETERMINE WORD COUNT */
		
		// Both max and min values
		if ($max != null
			&& ($wordCount <= $max && $wordCount >= $min))
		{
			return true;
		}
		// Only min value
		elseif ($max == null
			&& ($wordCount >= $min))
		{
			return true;
		}
		else
		{
			return false;
		}
	}
	
	/**
	 * Formulates a parameter-specific error message
	 * 
	 * @param  integer $min Minimum number of words required
	 * @param  integer $max Maximum number of words allowed
	 * @return string       Error message
	 */
	protected function _formulateErrorMessage($min, $max)
	{
		if ($min > 0 && $max > 0)
		{
			return sprintf(hasWordCount::RANGE_MESSAGE, $min, $max);
		}
		elseif ($min == 0 && $max > 0)
		{
			return sprintf(hasWordCount::MAX_MESSAGE, $max);
		}
		else
		{
			return sprintf(hasWordCount::MIN_MESSAGE, $min);
		}
	}
}