<?php
/**
 * Pancake Forms
 * 
 * @author     Ethan Poole
 * @package    Pancake
 * @subpackage Validators
 */

namespace Pancake\Validator;

use Pancake\Helper;

/**
 * MIME type validator
 * 
 * Examples:
 * <code>
 * $mimeType->validate('file.pdf', array('PDF' => 'application/pdf')); // True
 * $mimeType->validate('image.jpg', array('GIF' => 'image/gif', 'PNG' => 'image/png')); // Multiple allowed options; False
 * </code>
 * 
 * When forming the error message, isMIMEType looks at the array keys for the
 * name of the file type.  Some MIME types to names are pre-defined centrally
 * in {@link isMIMEType::$_typesToNames}.  If neither of these locations have
 * a file-type name, the MIME type is printed as a default.
 * 
 * @package    Pancake
 * @subpackage Validators
 */
class IsMIMEType extends Validator
{
	/**
	 * Dynamic error message template
	 */
	const ERROR_MESSAGE = 'Please upload a %s file.';
	
	/**
	 * Pre-defined MIME types to file-type names
	 * 
	 * @var array
	 */
	protected $_typesToNames = array(
		// Applications
		'application/msword' => 'Microsoft Word',
		'application/pdf' => 'PDF',
		'application/rtf' => 'RTF',
		'application/vnd.ms-excel' => 'Microsoft Excel',
		'application/vnd.ms-powerpoint' => 'Microsoft Powerpoint',
		'application/vnd.openxmlformats-officedocument.presentationml.presentation' => 'Microsoft Powerpoint',
		'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet' => 'Microsoft Excel',
		'application/vnd.openxmlformats-officedocument.wordprocessingml.document' => 'Microsoft Word',
		'application/zip' => 'Microsoft Word',
		'application/vnd.ms-office' => 'Microsoft Word',
		
		// Audio
		'audio/mpeg' => 'MP3',
		
		// Images
		'image/bmp' => 'Bitmap',
		'image/gif' => 'GIF',
		'image/jpeg' => 'JPEG',
		'image/png' => 'PNG',
		
		// Text
		'text/plain' => 'Text',
		'text/rtf' => 'RTF',
		
		// Video
		'video/quicktime' => 'QuickTime',
		'video/mpeg' => 'MPEG'
	);
	
	/**
	 * @param  mixed   $value
	 * @param  integer $mimeTypes Allowed MIME types
	 * @return boolean
	 */
	protected function _isTrue($value, $mimeTypes)
	{
		// Ensure the MIME types are stored in an array
		if (!is_array($mimeTypes))
		{
			$mimeTypes = array($mimeTypes);
		}
		
		// Set the dynamic error message
		$this->_error = $this->_formulateErrorMessage($mimeTypes);
		
		// Make sure the file exists
		if (!file_exists($value))
		{
			return false;
		}
		
		return in_array(mime_content_type($value), $mimeTypes);
	}
	
	/**
	 * Displays a list of allowed file types
	 * 
	 * @param  array $mimeTypes Allowed MIME types
	 * @return string
	 */
	protected function _formulateErrorMessage($mimeTypes)
	{
		$fileNames = array();
		foreach ($mimeTypes as $key => $value)
		{
			// File type as array key
			if (is_string($key))
			{
				$fileNames[] = $key;
			}
			// Centrally pre-defined file type
			elseif (isset($this->_typesToNames[$value]))
			{
				$fileNames[] = $this->_typesToNames[$value];
			}
			// Default to the MIME type
			else
			{
				$fileNames[] = $value;
			}
		}
		
		return sprintf(isMIMEType::ERROR_MESSAGE, Helper\SUA::joinAnd(', ', array_unique($fileNames), 'or'));
	}
}