<?php
/**
 * Pancake Forms
 * 
 * @author  Ethan Poole
 * @package Pancake
 */

namespace Pancake\Test;

use Pancake\Exception;
use Pancake\Field\Date;
use Pancake\Field\Field;
use Pancake\Field\InputText;

/**
 * {@link \Pancake\Field\Field} unit tests
 * 
 * @package    Pancake
 * @subpackage Tests
 */
class FieldTest extends Test
{
	function test()
	{
		$this->assertTrue(class_exists('Pancake\Field\Field'));
	}
}

/**
 * {@link \Pancake\Field\Field} unit tests for accessible text
 * 
 * @package    Pancake
 * @subpackage Tests
 */
class FieldAccessibleTest extends Test
{
	function testLabel()
	{
		$field = new FieldZ('test', array('accessible_text' => 'foobar'));
		$this->assertEqual($field->label(), '<label for="f_test">Test <span class="accessible">foobar</span></label>');
	}
	
	function testParagraphClass()
	{
		$field = new FieldZ('test', array('accessible_text' => 'foobar'));
		$this->assertEqual($field->__toString(), '<p class="accessible-text bar foo"><label for="f_test">Test <span class="accessible">foobar</span></label>kiitos</p>');
	}
}

/**
 * {@link \Pancake\Field\Field} unit tests for before and after text
 * 
 * @package    Pancake
 * @subpackage Tests
 */
class FieldBeforeAfterTest extends Test
{
	function testBefore()
	{
		$field = new FieldZ('test', array('before' => 'foo'));
		$this->assertEqual($field->__toString(), '<p class="bar foo">foo<label for="f_test">Test</label>kiitos</p>');
	}
	
	function testAfter()
	{
		$field = new FieldZ('test', array('after' => 'foo'));
		$this->assertEqual($field->__toString(), '<p class="bar foo"><label for="f_test">Test</label>kiitosfoo</p>');
	}
}

/**
 * {@link \Pancake\Field\Field} unit tests for child fields
 * 
 * @package    Pancake
 * @subpackage Tests
 */
class FieldChildTest extends Test
{
	function testArrayExistence()
	{
		$field = new Date('date');
		$this->assertIsA($field->childFields, 'array');
	}
	
	function testValidation()
	{
		$field = new Date('date');
		$field->value = time();
		$this->assertTrue($field->isValid());
	}
	
	function testInitialValue()
	{
		$field = new Date('date', array('value' => time()));
		$this->assertTrue($field->isValid());
	}
}

/**
 * {@link \Pancake\Field\Field} unit tests for the container element
 * 
 * @package    Pancake
 * @subpackage Tests
 */
class FieldContatinerTest extends Test
{
	function test()
	{
		$field = new FieldZ('test', array('container' => 'div'));
		$this->assertEqual($field->__toString(), '<div class="bar foo"><label for="f_test">Test</label>kiitos</div>');
	}
}

/**
 * {@link \Pancake\Field\Field::__construct()} unit tests
 * 
 * @package    Pancake
 * @subpackage Tests
 */
class FieldCreationTest extends Test
{
	function test()
	{
		$field = new RequiredField('name', 'name', array('paragraph' => true));
		
		$this->assertEqual($field->name, 'name');
		$this->assertEqual($field->property('paragraph'), true);
	}
	
	function testAttributesAndParamtersAreSet()
	{
		$field = new FieldZ('test', array(
			'class' => 'foobar',
			'paragraph' => true
		));
		
		$this->assertEqual($field->attribute('class'), 'foobar');
		$this->assertEqual($field->property('paragraph'), true);
	}
	
	function testBadNames()
	{
		try
		{
			$field = new FieldZ('foo bar');
			$this->fail('An exception was expected.');
		}
		catch (Exception\ValueError $e)
		{
			$this->pass();
		}
		
		try
		{
			$field = new FieldZ('foobar*');
			$this->fail('An exception was expected.');
		}
		catch (Exception\ValueError $e)
		{
			$this->pass();
		}
	}
	
	function testExtraParameters()
	{
		try
		{
			$field = new FieldZ('test', array(
				'validation' => 'test_validation',
				'foo' => 'bar'
			));
			$this->fail('An exception was expected.');
		}
		catch (Exception\Exception $e)
		{
			$this->pass();
		}
	}
	
	function testRequiredPropertyIsSet()
	{
		$field = new RequiredFieldAlt('name', false);
		$this->assertFalse($field->property('required'));
	}
	
	function testTooFewParameters()
	{
		try
		{
			$field = new RequiredField;
			$this->fail('An exception was expected.');
		}
		catch (Exception\Exception $e)
		{
			$this->pass();
		}
	}
	
	function testTooManyParameters()
	{
		try
		{
			$field = new RequiredField('test', 'test', 'test');
			$this->fail('An exception was expected.');
		}
		catch (Exception\Exception $e)
		{
			$this->pass();
		}
	}
	
	function testWrongDataType()
	{
		try
		{
			$field = new RequiredField(array('test'));
			$this->fail('An exception was expected.');
		}
		catch (Exception\MissingArgument $e)
		{
			$this->pass();
		}
	}
}

/**
 * {@link \Pancake\Field\Field::_formattedValue()} unit tests
 * 
 * @package    Pancake
 * @subpackage Tests
 */
class FieldFormattedValueTest extends Test
{
	function test()
	{
		$field = new InputText('foobar');
		$field->value = 'test';
		$this->assertEqual($field->formattedValue, 'test');
	}
}

/**
 * {@link \Pancake\Field\Field::__get()} and {@link \Pancake\Field\Field::__set()} unit tests
 * 
 * @package    Pancake
 * @subpackage Tests
 */
class FieldGettersSettersTest extends Test
{
	function testGet()
	{
		$field = new FieldZ('test');
		
		$this->assertEqual($field->name, 'test');
		$this->assertEqual($field->errors, array());
		$this->assertEqual($field->canBeFormIdentifier, false);
		$this->assertEqual($field->childFields, array());
		$this->assertEqual($field->value, null);
		$this->assertEqual($field->formattedValue, null);
		$this->assertEqual($field->id, 'f_test');
		$this->assertEqual($field->label, '<label for="f_test">Test</label>');
	}
	
	function testSet()
	{
		$field = new FieldZ('test');
		
		$field->value = 'baz';
		$this->assertEqual($field->value, 'baz');
	}
}

/**
 * {@link \Pancake\Field\Field} unit tests for help text
 * 
 * @package    Pancake
 * @subpackage Tests
 */
class FieldHelpTextTest extends Test
{
	function test()
	{
		$field = new FieldZ('test', array('help_text' => 'foobar'));
		$this->assertEqual($field->label(), '<label for="f_test">Test <small class="note">foobar</small></label>');
	}
	
	function testParagraphClass()
	{
		$field = new FieldZ('test', array('help_text' => 'foobar'));
		$this->assertEqual($field->__toString(), '<p class="bar foo help-text"><label for="f_test">Test <small class="note">foobar</small></label>kiitos</p>');
	}
}

/**
 * {@link \Pancake\Field\Field::label()} unit tests
 * 
 * @package    Pancake
 * @subpackage Tests
 */
class FieldLabelTest extends Test
{
	function test()
	{
		$field = new FieldZ('test');
		$this->assertEqual($field->label, '<label for="f_test">Test</label>');
	}
	
	function testAttributes()
	{
		$field = new FieldZ('test');
		$this->assertEqual($field->label(null, true, array('class' => 'foo bar', 'for' => 'baz')), '<label class="foo bar" for="baz">Test</label>');
	}
	
	function testComplexText()
	{
		$field = new FieldZ('sven_the_viking');
		$this->assertEqual($field->label, '<label for="f_sven_the_viking">Sven the Viking</label>');
	}
	
	function testCustomLabelTag()
	{
		$field = new FieldZ('test', array('label_element' => 'h2'));
		$this->assertEqual($field->label(), '<h2>Test</h2>');
	}
	
	function testCustomText()
	{
		$field = new FieldZ('test');
		$this->assertEqual($field->label('FooBar'), '<label for="f_test">FooBar</label>');
	}
	
	function testErrorMessage()
	{
		$field = new FieldZ('sven_the_viking', array('validation' => '\Pancake\Test\test_validation_error'));
		$field->isValid();
		$this->assertPattern('#<label for="f_sven_the_viking">Sven the Viking <strong class="error">(.+?)</strong></label>#', $field->label);
	}
	
	function testHideErrorMessage()
	{
		$field = new FieldZ('sven_the_viking', array('validation' => '\Pancake\Test\test_validation_error', 'show_errors' => false));
		$field->isValid();
		$this->assertPattern('#<label for="f_sven_the_viking">Sven the Viking</label>#', $field->label);
	}
	
	function testMultipleErrorMessages()
	{
		$field = new FieldZ('sven_the_viking', array('validation' => array('\Pancake\Test\test_validation_error', '\Pancake\Test\test_validation_error')));
		$field->isValid();
		$this->assertPattern('#<label for="f_sven_the_viking">Sven the Viking <strong class="error">(.+?)</strong> <strong class="error">(.+?)</strong></label>#', $field->label);
	}
	
	function testMultipleWords()
	{
		$field = new FieldZ('foo_bar');
		$this->assertEqual($field->label, '<label for="f_foo_bar">Foo Bar</label>');
	}
	
	function testOnlyText()
	{
		$field = new FieldZ('test');
		$this->assertEqual($field->label(null, false), 'Test');
	}
	
	function testSetViaProperty()
	{
		$field = new FieldZ('test', array('label' => 'FooBar'));
		$this->assertEqual($field->label(), '<label for="f_test">FooBar</label>');
	}
}

/**
 * {@link \Pancake\Field\Field} unit tests for optionality message
 * 
 * @package    Pancake
 * @subpackage Tests
 */
class FieldOptionalityTest extends Test
{
	function test()
	{
		$field = new FieldZ('test', array('required' => false));
		$this->assertEqual($field->label(), '<label for="f_test">Test <em class="optional">(optional)</em></label>');
	}
	
	function testNoDisplay()
	{
		$field = new FieldZ('test', array('required' => false, 'show_if_optional' => false));
		$this->assertEqual($field->label(), '<label for="f_test">Test</label>');
	}
	
	function testParagraphClass()
	{
		$field = new FieldZ('test', array('required' => false));
		$this->assertEqual($field->__toString(), '<p class="bar foo optional"><label for="f_test">Test <em class="optional">(optional)</em></label>kiitos</p>');
	}
}

/**
 * {@link \Pancake\Field\Field::output()} unit tests
 * 
 * @package    Pancake
 * @subpackage Tests
 */
class FieldOutputTest extends Test
{
	function test()
	{
		$field = new FieldZ('test');
		$this->assertEqual($field->output(), 'kiitos');
	}
}

/**
 * {@link \Pancake\Field\Field::property()} and {@link \Pancake\Field\Field::attribute()} unit tests
 * 
 * @package    Pancake
 * @subpackage Tests
 */
class FieldPropertyAttributesTest extends Test
{
	function testPropertyGetter()
	{
		$field = new FieldZ('test');
		$this->assertIsA($field->property('paragraph'), 'boolean');
		
		try
		{
			$field->property('foobar');
			$this->fail('Exception was expected.');
		}
		catch (Exception\DoesNotExist $e)
		{
			$this->pass();
		}
	}
	
	function testPropertySetter()
	{
		$field = new FieldZ('test');
		$field->property('paragraph', false);
		$this->assertFalse($field->property('paragraph'));
	}
	
	function testAttributeGetter()
	{
		$field = new FieldZ('test', array('class' => 'foobar'));
		$this->assertEqual($field->attribute('class'), 'foobar');
		
		try
		{
			$field->attribute('foobar');
			$this->fail('Exception was expected.');
		}
		catch (Exception\DoesNotExist $e)
		{
			$this->pass();
		}
	}
	
	function testAttributeSetter()
	{
		$field = new FieldZ('test');
		$field->attribute('class', 'foobar');
		$this->assertEqual($field->attribute('class'), 'foobar');
	}
}

/**
 * {@link \Pancake\Field\Field::__toString()} unit tests
 * 
 * @package    Pancake
 * @subpackage Tests
 */
class FieldToStringTest extends Test
{
	function testParagraph()
	{
		$field = new FieldZ('test');
		$this->assertEqual($field->__toString(), '<p class="bar foo"><label for="f_test">Test</label>kiitos</p>');
	}
	
	function testNoParagraph()
	{
		$field = new FieldZ('test', array(
			'paragraph' => false
		));
		$this->assertEqual($field->__toString(), '<label for="f_test">Test</label>kiitos');
	}
	
	function testParagraphClasses()
	{
		$field = new FieldZ('test', array(
			'paragraph_classes' => array('baz', 'qux')
		));
		$this->assertEqual($field->__toString(), '<p class="bar baz foo qux"><label for="f_test">Test</label>kiitos</p>');
	}
	
	function testNoLabel()
	{
		$field = new FieldZ('test', array(
			'show_label' => false
		));
		$this->assertEqual($field->__toString(), '<p class="bar foo">kiitos</p>');
	}
	
	function testNoField()
	{
		$field = new FieldZ('test', array(
			'show_field' => false
		));
		$this->assertEqual($field->__toString(), '<p class="bar foo"><label for="f_test">Test</label></p>');
	}
	
	function testNoLabelNoParagraph()
	{
		$field = new FieldZ('test', array(
			'paragraph' => false,
			'show_label' => false
		));
		$this->assertEqual($field->__toString(), 'kiitos');
	}
	
	function testErrorMessage()
	{
		$field = new FieldZ('sven_the_viking', array('validation' => '\Pancake\Test\test_validation_error'));
		$field->isValid();
		$this->assertPattern('#<p class="bar error foo"><label for="f_sven_the_viking">Sven the Viking <strong class="error">(.+?)</strong></label>kiitos</p>#', $field->__toString());
	}
}

/**
 * {@link \Pancake\Field\Field::isValid()} unit tests
 * 
 * @package    Pancake
 * @subpackage Tests
 */
class FieldValidationTest extends Test
{
	function test()
	{
		$fieldTrue = new FieldZ('test', array(
			'validation' => '\Pancake\Test\test_validation',
			'value' => 'string'
		));
		
		$fieldFalse = new FieldZ('test', array(
			'validation' => '\Pancake\Test\test_validation_error',
			'value' => array('foo', 'bar')
		));
		
		$this->assertTrue($fieldTrue->isValid());
		$this->assertFalse($fieldFalse->isValid());
	}
	
	function testErrorReturn()
	{
		$field = new FieldZ('test', array(
			'validation' => '\Pancake\Test\test_validation_error',
			'value' => array('foo', 'bar')
		));
		
		$this->assertFalse($field->isValid());
		$this->assertEqual($field->errors, array('Error'));
	}
	
	function testLibraryValidator()
	{
		$field = new FieldZ('test', array(
			'validation' => 'isNumeric',
			'value' => '666'
		));
		
		$this->assertTrue($field->isValid());
	}
	
	function testLibraryValidatorParameters()
	{
		$field = new FieldZ('test', array(
			'validation' => array(array('isBetween', 1, 6)),
			'value' => '5'
		));
		
		$this->assertTrue($field->isValid());
	}
	
	function testMultiple()
	{
		$field = new FieldZ('test', array(
			'validation' => array('isNumeric', array('isBetween', 1, 10)),
			'value' => '666'
		));
		
		$this->assertFalse($field->isValid());
	}
		
	function testNonExistentFunction()
	{
		try
		{
			$field = new FieldZ('test', array(
				'validation' => 'blah',
				'value' => '8'
			));
			
			$this->fail('An exception was expected.');
		}
		catch (Exception\Exception $e)
		{
			$this->pass();
		}
	}
	
	function testStoreErrorMessage()
	{
		$field = new FieldZ('test', array(
			'validation' => array(array('isBetween', 1, 6, 'error' => 'Foobar')),
			'value' => '666'
		));
		
		$this->assertFalse($field->isValid(false));
		$this->assertEqual($field->errors, array());
	}
	
	function testValidatorErrorMessages()
	{
		$field = new FieldZ('test', array(
			'validation' => array(array('isBetween', 1, 6, 'error' => 'Foobar')),
			'value' => '666'
		));
		
		$this->assertFalse($field->isValid());
		$this->assertEqual($field->errors, array('Foobar'));
	}
}

/* STUFF FOR TESTING */
	/**
	 * @package    Pancake
	 * @subpackage UnitTests
	 */
	class FieldZ extends Field
	{
		protected $_paragraphClasses = array('foo', 'bar');
		protected $_validAttributes = array('class' => null);
		function output() { return 'kiitos'; }
	}
	
	/**
	 * @package    Pancake
	 * @subpackage UnitTests
	 */
	class RequiredField extends FieldZ
	{
		protected $_requiredParameters = array('class');
	}
	
	/**
	 * @package    Pancake
	 * @subpackage UnitTests
	 */
	class RequiredFieldAlt extends FieldZ
	{
		protected $_requiredParameters = array('required');
	}
	
	function test_validation($string) { return is_string($string); }
	function test_validation_error($string) { return 'Error'; }
/* END STUFF FOR TESTING */