<?php
/**
 * Pancake Forms
 * 
 * @author  Ethan Poole
 * @package Pancake
 */

namespace Pancake\Test;

use Pancake\Exception;
use Pancake\Loader\FieldLoader;

/**
 * General {@link \Pancake\Loader\FieldLoader} unit tests
 * 
 * @package    Pancake
 * @subpackage Tests
 */
class FieldLoaderTest extends Test
{
	protected $_fieldDirectory;
	
	function setUp()
	{
		$this->_fieldDirectory = array('\Pancake\Field\\' => PANCAKE_ROOT . 'Field');
	}
	
	function test()
	{
		$this->assertTrue(class_exists('\Pancake\Loader\FieldLoader'));
	}
	
	function testArrayLike()
	{
		$loader = new FieldLoader($this->_fieldDirectory);
		
		$loader->load('text', array('foobar1'));
		$loader->load('text', array('foobar2'));
		$loader->load('text', array('foobar3'));
		
		// Test countable
		$this->assertEqual(count($loader), 3);
		
		// Test iterable
		foreach ($loader as $field)
		{
			$this->assertTrue(is_a($field, '\Pancake\Field\Field'));
		}
		
		// Test array-accessible
		$loader['foobar1']->value = 'test';
		$this->assertEqual($loader['foobar1']->value, 'test');
		$test = $loader['foobar1'];
		$test->value = 'test2';
		$this->assertEqual($loader['foobar1']->value, 'test2');
		$loader['foobar1']->value = 'test3';
		$this->assertEqual($loader['foobar1']->value, 'test3');
		$this->assertEqual($test->value, 'test3');
	}
	
	function testConstrutorRequiresDirectories()
	{
		try
		{
			$loader = new FieldLoader(array('foobar'));
			$this->fail('An exception was expected.');
		}
		catch (Exception\FileError $e)
		{
			$this->pass();
		}
	}
}

/**
 * {@link \Pancake\Loader\FieldLoader::load()} unit tests
 * 
 * @package    Pancake
 * @subpackage Tests
 */
class FieldLoaderLoadTest extends Test
{
	protected $_fieldDirectory;
	
	function setUp()
	{
		$this->_fieldDirectory = array('\Pancake\Field\\' => PANCAKE_ROOT . 'Field');
	}
	
	function testClassName()
	{
		$loader = new FieldLoader($this->_fieldDirectory);
		$field = $loader->load('\Pancake\Field\InputText', array('foobar2'));
		$this->assertIsA($field, '\Pancake\Field\InputText');
		$this->assertEqual($field->name, 'foobar2');
	}
	
	function testDuplicateNames()
	{
		$loader = new FieldLoader($this->_fieldDirectory);
		
		try
		{
			$field = $loader->load('text', array('foobar'));
			$field = $loader->load('text', array('foobar'));
			$this->fail('An exception was expected.');
		}
		catch (Exception\NameError $e)
		{
			$this->pass();
		}
	}
	
	function testInvalidField()
	{
		$loader = new FieldLoader($this->_fieldDirectory);
		
		try
		{
			$fieldC = $loader->load('foobar');
			$this->fail('An exception was expected.');
		}
		catch (Exception\NameError $e)
		{
			$this->pass();
		}
	}
	
	function testMethodName()
	{
		$loader = new FieldLoader($this->_fieldDirectory);
		$field = $loader->load('text', array('foobar'));
		$this->assertIsA($field, '\Pancake\Field\InputText');
		$this->assertEqual($field->name, 'foobar');
	}
	
	function testNoParameters()
	{
		$loader = new FieldLoader($this->_fieldDirectory);
		
		try
		{
			$fieldC = $loader->load('text');
			$this->fail('An exception was expected.');
		}
		catch (\Exception $e)
		{
			$this->pass();
		}
	}
}

/**
 * {@link \Pancake\Loader\FieldLoader::loadAll()} unit tests
 * 
 * @package    Pancake
 * @subpackage Tests
 */
class FieldLoaderLoadAllTest extends Test
{
	protected $_fieldDirectory;
	protected $_noClassDirectory;
	protected $_duplicateMethodsDirectory;
	protected $_expectedFields = 19;
	
	function setUp()
	{
		$this->_fieldDirectory = array('\Pancake\Field\\' => PANCAKE_ROOT . 'Field');
		$this->_noClassDirectory = array(PANCAKE_ROOT . '../tests-simpletest/fields/no-class');
		$this->_duplicateMethodsDirectory = array(PANCAKE_ROOT . '../tests-simpletest/fields/duplicate-methods');
	}
	
	function testConstructorFieldLoading()
	{
		$loader = new FieldLoader($this->_fieldDirectory, false);
		$this->assertEqual(count($loader->fields), 0);
		
		$loader->loadAll();
		$this->assertEqual(count($loader->validFields), $this->_expectedFields);
	}
	
	function testDuplicateMethodNames()
	{
		try
		{
			$loader = new FieldLoader($this->_duplicateMethodsDirectory);
			$this->fail('An exception was expected.');
		}
		catch (Exception\NameError $e)
		{
			$this->pass();
		}
	}
	
	function testNoCorrespondingClass()
	{
		try
		{
			$loader = new FieldLoader($this->_noClassDirectory);
			$this->fail('An exception was expected.');
		}
		catch (Exception\NameError $e)
		{
			$this->pass();
		}
	}
}