<?php
/**
 * Pancake Forms
 * 
 * @author  Ethan Poole
 * @package Pancake
 */

namespace Pancake\Test;

use Pancake\Exception;
use Pancake\Pancake;

/**
 * General {@link \Pancake\PancakeAbstract} unit tests
 * 
 * @package    Pancake
 * @subpackage Tests
 */
class PancakeAbstractTest extends Test
{
	function test()
	{
		$this->assertTrue(class_exists('Pancake\PancakeAbstract'));
	}
	
	function testStringReplacement()
	{
		echo $this->_pancake->start();
			echo $this->_pancake->text('foo');
		$buffer = $this->_pancake->end();
		
		$this->assertPattern('#<form action="' . $_SERVER['REQUEST_URI'] . '" class="pancake" method="post"><div class="hide"><input name="csrf_token" type="hidden" value="(.+)" /><p><label for="f_honeypot">If you enter anything in this field your comment will be treated as spam:</label><input id="f_honeypot" name="honeypot" type="text" /></p></div><p class="field text"><label for="f_foo">Foo</label><input id="f_foo" name="foo" type="text" /></p></form>#m', str_replace(array("\n", "\t"), null, $buffer));
	}
	
	function testGetField()
	{
		$this->_pancake->addField('text', array('foo'));
		$field = $this->_pancake->foo;
		$this->assertEqual(get_class($field), 'Pancake\Field\InputText');
	}
}

/**
 * {@link \Pancake\PancakeAbstract::addField()} unit tests
 * 
 * @package    Pancake
 * @subpackage Tests
 */
class PancakeAbstractAddFieldTest extends Test
{
	function test()
	{
		$field = $this->_pancake->addField('text', array('test'));
		$this->assertEqual($field, '{{ Field: test }}');
		
		// Ensure that it is running through FieldLoader
		try
		{
			$this->_pancake->addField('foobar', array('test'));
			$this->fail('An exception was expected.');
		}
		catch (Exception\NameError $e)
		{
			$this->pass();
		}
	}
	
	function testMagicMethodCall()
	{
		$field = $this->_pancake->text('foobar');
		$this->assertEqual($field, '{{ Field: foobar }}');
		$this->assertEqual(get_class($this->_pancake->foobar), 'Pancake\Field\InputText');
	}
	
	function testPresetValue()
	{
		$this->_pancake->presetValue('foo', 'bar');
		$this->assertTrue($this->_pancake->addField('text', array('foo')));
		$this->assertEqual($this->_pancake->foo->value, 'bar');
	}
	
	function testSetIdentifier()
	{
		$this->_pancake->addField('submit', array('submit', 'Submit'));
		$this->assertEqual($this->_pancake->setting('identifier'), 'submit');
	}
}

/**
 * {@link \Pancake\PancakeAbstract::end()} unit tests
 * 
 * @package    Pancake
 * @subpackage Tests
 */
class PancakeAbstractEndTest extends Test
{
	function test()
	{
		ob_start();
		$this->assertEqual($this->_pancake->end(), '</form>');
	}
}

/**
 * {@link \Pancake\PancakeAbstract::errors()} unit tests
 * 
 * @package    Pancake
 * @subpackage Tests
 */
class PancakeAbstractErrorsTest extends Test
{
	function testHideErrors()
	{
		$this->_pancake->addField('text', array('name', array('value' => 'name')));
		$this->_pancake->addField('text', array('number', array('value' => 'string', 'validation' => 'isNumeric', 'show_errors' => false)));
		$this->_pancake->isValid();
		
		$this->assertNull($this->_pancake->errors);
	}
	
	function testInvalid()
	{
		$this->_pancake->addField('text', array('name', array('value' => 'name')));
		$this->_pancake->addField('text', array('number', array('value' => 'string', 'validation' => 'isNumeric')));
		$this->_pancake->isValid();
		
		$this->assertPattern('#<div class="message error"><h3>Form Not Yet Submitted!</h3><p>Sorry, but your form was not submitted!  Please correct the following errors and submit the form again:</p><ul><li><span class="field">Number</span>: (.+?)</li></ul></div>#', $this->_pancake->errors);
	}
	
	function testJustArray()
	{
		$this->_pancake->addField('text', array('name', array('value' => 'name')));
		$this->_pancake->addField('text', array('number', array('value' => 'string', 'validation' => 'isNumeric')));
		$this->_pancake->isValid();
		
		$this->assertIsA($this->_pancake->errors(false), 'array');
	}
	
	function testShortLabels()
	{
		$this->_pancake->addField('text', array('number', array('validation' => 'isNumeric', 'label' => 'FooBar', 'label_short' => 'Foo')));
		$this->_pancake->isValid();
		
		$this->assertPattern('#<div class="message error"><h3>Form Not Yet Submitted!</h3><p>Sorry, but your form was not submitted!  Please correct the following errors and submit the form again:</p><ul><li><span class="field">Foo</span>: (.+?)</li></ul></div>#', $this->_pancake->errors);
	}
	
	function testValid()
	{
		$this->_pancake->addField('text', array('name', array('value' => 'name')));
		$this->_pancake->addField('text', array('number', array('value' => '123', 'validation' => 'isNumeric')));
		$this->_pancake->isValid();
		
		$this->assertNull($this->_pancake->errors);
	}
}

/**
 * {@link \Pancake\PancakeAbstract::setting()} unit tests
 * 
 * @package    Pancake
 * @subpackage Tests
 */
class PancakeAbstractSettingsTest extends Test
{
	function testGet()
	{
		$this->assertIsA($this->_pancake->setting('directory_fields'), 'string');
	}
	
	function testNonExistentSetting()
	{
		try
		{
			$this->_pancake->setting('foobar', 'foobar');
			$this->fail('Exception was expected.');
		}
		catch (Exception\DoesNotExist $e)
		{
			$this->pass();
		}
	}
	
	function testSet()
	{
		$this->_pancake->setting('directory_fields', 'foobar');
		$this->assertEqual($this->_pancake->setting('directory_fields'), 'foobar');
	}
	
	function testSetAtRuntime()
	{
		ob_start();
		$pancake = new Pancake(array(
			'directions' => 'baz'
		));
		$buffer = ob_get_clean();
		
		$this->assertNotNull($pancake->setting('directions'));
		$this->assertEqual($pancake->setting('directions'), 'baz');
	}
}

/**
 * {@link \Pancake\PancakeAbstract::start()} unit tests
 * 
 * @package    Pancake
 * @subpackage Tests
 */
class PancakeAbstractStartTest extends Test
{
	function test()
	{
		$this->assertPattern('#<form action="' . $_SERVER['REQUEST_URI'] . '" class="pancake" method="post"><div class="hide"><input name="csrf_token" type="hidden" value="(.+?)" /><p><label for="f_honeypot">If you enter anything in this field your comment will be treated as spam:</label><input id="f_honeypot" name="honeypot" type="text" /></p></div>#', str_replace(array("\n", "\t"), null, $this->_pancake->start()));
		ob_end_flush();
	}
	
	function testCustomStart()
	{
		$this->assertPattern('#<form action="http://www.google.com" class="foobar" method="get"><div class="hide"><input name="csrf_token" type="hidden" value="(.+?)" /><p><label for="f_honeypot">If you enter anything in this field your comment will be treated as spam:</label><input id="f_honeypot" name="honeypot" type="text" /></p></div>#', str_replace(array("\n", "\t"), null, $this->_pancake->start(array(
			'action' => 'http://www.google.com',
			'class' => 'foobar',
			'method' => 'get'
		))));
		ob_end_flush();
	}
	
	function testInvalidStartAttributes()
	{
		try
		{
			$this->_pancake->start(array(
				'action' => 'http://www.google.com',
				'method' => 'get',
				'foo' => 'bar'
			));
			
			$this->fail('An exception was expected.');
		}
		catch (Exception\Exception $e)
		{
			$this->pass();
		}
		
		ob_end_flush();
	}
}

/**
 * {@link \Pancake\PancakeAbstract::isValid()} unit tests
 * 
 * @package    Pancake
 * @subpackage Tests
 */
class PancakeAbstractValidationTest extends Test
{
	function testInvalid()
	{
		$this->_pancake->addField('text', array('name', array('value' => 'name')));
		$this->_pancake->addField('text', array('number', array('value' => 'string', 'validation' => 'isNumeric')));
		
		$this->assertFalse($this->_pancake->isValid());
	}
	
	function testRequired()
	{
		$this->_pancake->addField('text', array('name'));
		$this->assertFalse($this->_pancake->isValid());
	}
	
	function testValid()
	{
		$this->_pancake->addField('text', array('name', array('value' => 'name')));
		$this->_pancake->addField('text', array('number', array('value' => '123', 'validation' => 'isNumeric')));
		
		$this->assertTrue($this->_pancake->isValid());
	}	
}